@@ -9,6 +9,23 @@
 //  Module:         Application configuration data
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
 #include "common.h"
 
 #include <limits.h>
@@ -19,8 +36,8 @@
 
 #include "libewf.h"
 
-#include "cfg/toolcfg.h"
-#include "sysinfo/toolsysinfo.h"
+#include "toolcfg.h"
+#include "toolsysinfo.h"
 #include "file.h"
 
 #include "qtutil.h"
@@ -108,7 +125,6 @@ static APIRET CfgDlgAcquireRuleEnd                                            (t
 static APIRET CfgIniLang (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText);
 static APIRET CfgIniCPUs (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText);
 static APIRET CfgIniMem  (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText);
-static APIRET CfgIniFifo (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText);
 
 // ------------------------------------
 //     Configuration parameter table
@@ -260,6 +276,22 @@ static t_ToolCfgSet SetArrCompressionThreads[] =
 };
 
 
+static t_ToolCfgSet SetArrDeviceScanMethod[] =
+{
+   // Name in cfg file  Corresponding value
+   // ----------------------------------------
+   { "LibParted",  SCANMETHOD_LIBPARTED },
+   { "Parted",     SCANMETHOD_LIBPARTED },
+   { "DBusHAL",    SCANMETHOD_DBUSHAL   },
+   { "HAL",        SCANMETHOD_DBUSHAL   },
+   { "DBusDevKit", SCANMETHOD_DBUSDEVKIT},
+   { "DevKit",     SCANMETHOD_DBUSDEVKIT},
+   { "DBusUDisks", SCANMETHOD_DBUSDEVKIT},
+   { "UDisks",     SCANMETHOD_DBUSDEVKIT},
+   {  NULL,        0                    }
+};
+
+
 #define ELT(Elt)      ((long) &CfgLocal.CfgData.Elt)
 #define ELT_SIZ(Elt)  ELT(Elt), (sizeof (CfgLocal.CfgData.Elt)-1)                      // for strings only, thus substract 1 byte (for terminating 0)
 #define INIARR(Arr)   ((t_pToolCfgSet) (t_pvoid)&Arr)
@@ -291,22 +323,24 @@ static t_ToolCfgParamDesc CfgParamDescArr[] =
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"ScreenRefreshInterval"         , CFGTYPE_INTEGER, ELT(ScreenRefreshInterval         ),                0,       1, 8640000, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"UseFileDialogFromQt"           , CFGTYPE_SET    , ELT(UseFileDialogFromQt)           ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
 
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"AutoExit"                      , CFGTYPE_SET    , ELT(AutoExit)                      ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"AutoExitCountdown"             , CFGTYPE_INTEGER, ELT(AutoExitCountdown)             ,                0,       3,  864000, NULL                            }, CFG_FILLUP_FORLINT},
+
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"DefaultFormat"                 , CFGTYPE_SET    , ELT(DefaultFormat)                 ,                0,       0,       0, INIARR(SetArrFormat)            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"EwfFormat"                     , CFGTYPE_SET    , ELT(EwfFormat     )                ,                0,       0,       0, INIARR(SetArrEwfFormat     )    }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"EwfCompression"                , CFGTYPE_SET    , ELT(EwfCompression)                ,                0,       0,       0, INIARR(SetArrEwfCompression)    }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"AffCompression"                , CFGTYPE_INTEGER, ELT(AffCompression)                ,                0,       1,       9, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"AffMarkBadSectors"             , CFGTYPE_SET    , ELT(AffMarkBadSectors)             ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
-   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"EwfSegmentSize"                , CFGTYPE_INTEGER, ELT(EwfSegmentSize)                ,                0,       1,    2047, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"SpecialFilenameChars"          , CFGTYPE_STRING , ELT_SIZ(SpecialFilenameChars)      ,                         0,       0, NULL                            }, CFG_FILLUP_FORLINT},
 
-   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"ScanUsingDbusHal"              , CFGTYPE_SET    , ELT(ScanUsingDbusHal)              ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"DeviceScanMethod"              , CFGTYPE_SET    , ELT(DeviceScanMethod)              ,                0,       0,       0, INIARR(SetArrDeviceScanMethod)  }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"ScanInterval"                  , CFGTYPE_INTEGER, ELT(ScanInterval                  ),                0,       1, 8640000, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"CommandGetSerialNumber"        , CFGTYPE_STRING , ELT_SIZ(CommandGetSerialNumber    ),                         0,       0, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeDD"               , CFGTYPE_INTEGER, ELT(FifoBlockSizeDD               ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeEWF"              , CFGTYPE_INTEGER, ELT(FifoBlockSizeEWF              ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoBlockSizeAFF"              , CFGTYPE_INTEGER, ELT(FifoBlockSizeAFF              ),                0,       0,99999999, NULL                            }, CFG_FILLUP_FORLINT},
-   {CFGASSIGN_BOTH_MULTIPLE, CfgIniMem  , {"FifoMaxMem"                    , CFGTYPE_INTEGER, ELT(FifoMaxMem                    ),                0,       0, 1000000, NULL                            }, CFG_FILLUP_FORLINT},
-   {CFGASSIGN_BOTH_MULTIPLE, CfgIniFifo , {"FifoMaxEntries"                , CFGTYPE_INTEGER, ELT(FifoMaxEntries                ),                0,       0, 9999999, NULL                            }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, CfgIniMem  , {"FifoMaxMem"                    , CFGTYPE_INTEGER, ELT(FifoMaxMem                    ),                0,       0,    2000, NULL                            }, CFG_FILLUP_FORLINT},
+   {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"FifoMemoryManager"             , CFGTYPE_SET    , ELT(FifoMemoryManager)             ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"UseSeparateHashThread"         , CFGTYPE_SET    , ELT(UseSeparateHashThread)         ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, CfgIniCPUs , {"CompressionThreads"            , CFGTYPE_SET    , ELT(CompressionThreads)            ,                0,       0,    1024, INIARR(SetArrCompressionThreads)}, CFG_FILLUP_FORLINT},
    {CFGASSIGN_BOTH_MULTIPLE, NULL       , {"SignalHandling"                , CFGTYPE_SET    , ELT(SignalHandling)                ,                0,       0,       0, INIARR(SetArrBoolean)           }, CFG_FILLUP_FORLINT},
@@ -434,13 +468,14 @@ static t_pCfgBuffDlgAcquireField pFld0 = NULL;
 
 static t_ToolCfgDataDesc CfgDescArrDlgAcquireField[] =
 {
-   // ParameterName   Type (see        DestinationOfs    Len  Min  Max  SetArray
-   // in cfg file     t_ToolCfgType)   type is long                     (CFGTYPE_SET only)
-   // ------------------------------------------------------------------------------------
-   {  "FieldName"   , CFGTYPE_STRING , ELT_SIZ(FieldName)   ,   0,   0, NULL           },
-   {  "EntryMode"   , CFGTYPE_SET    , ELT(EntryMode)   ,  0,   0,   0, SetArrEntryMode},
-   {  "DefaultValue", CFGTYPE_STRING , ELT_SIZ(DefaultValue),   0,   0, NULL           },
-   {  NULL          , CFGTYPE_NULL   , 0                ,  0,   0,   0, NULL           }
+   // ParameterName     Type (see        DestinationOfs     Len  Min  Max  SetArray
+   // in cfg file       t_ToolCfgType)   type is long                      (CFGTYPE_SET only)
+   // ---------------------------------------------------------------------------------------
+   {  "FieldName"     , CFGTYPE_STRING , ELT_SIZ(FieldName)     ,  0,   0, NULL           },
+   {  "EntryModeImage", CFGTYPE_SET    , ELT(EntryModeImage),  0,  0,   0, SetArrEntryMode},
+   {  "EntryModeClone", CFGTYPE_SET    , ELT(EntryModeClone),  0,  0,   0, SetArrEntryMode},
+   {  "DefaultValue"  , CFGTYPE_STRING , ELT_SIZ(DefaultValue)  ,  0,   0, NULL           },
+   {  NULL            , CFGTYPE_NULL   , 0                  ,  0,  0,   0, NULL           }
 };
 
 
@@ -539,7 +574,7 @@ APIRET CfgIniLang (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText)
    if (Language.compare ("AUTO", Qt::CaseInsensitive) == 0)
    {
       QString LocaleName = QLocale::system().name();
-      QString Language = LocaleName.split('_')[0];
+      Language = LocaleName.split('_')[0];
       snprintf (CONFIG(Language), sizeof(CONFIG(Language)), "%s", QSTR_TO_PSZ(Language));
 
       LOG_INFO ("Parameter %s set to 'AUTO', switching to language '%s'", pCfgParamDesc->DataDesc.pName, CONFIG(Language))
@@ -579,26 +614,10 @@ APIRET CfgIniMem (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText)
    meminfo();
    if (CONFIG (FifoMaxMem) == 0)
    {
-      CONFIG (FifoMaxMem) = std::max (1, (int)(kb_main_total / (6*1024))); // Use one sixth of the available mem, convert KB to MB
+      CONFIG (FifoMaxMem) = GETMAX (1, (int)(kb_main_total / (8*1024)));   // Use one eighth of the available mem, convert KB to MB
+      CONFIG (FifoMaxMem) = GETMIN (CONFIG (FifoMaxMem), 64);              // Stay below 64MB
       LOG_INFO ("Parameter %s set to 0 (auto); %lu MB of RAM detected.", pParamName, kb_main_total/1024)
-      LOG_INFO ("Setting %s to %d.", pParamName, CONFIG (FifoMaxMem))
-   }
-
-   return NO_ERROR;
-}
-
-APIRET CfgIniFifo (t_pToolCfgParamDesc pCfgParamDesc, t_pchar *ppErrorText)
-{
-   t_pcchar pParamName;
-
-   *ppErrorText = NULL;
-   pParamName = pCfgParamDesc->DataDesc.pName;
-
-   meminfo();
-   if (CONFIG (FifoMaxEntries) == 0)
-   {
-      CONFIG (FifoMaxEntries) = 8;
-      LOG_INFO ("Parameter %s set to 0 (auto); setting it to %d.", pParamName, CONFIG (FifoMaxEntries))
+      LOG_INFO ("Setting %s to %d (i.e. using %d MB per acquisition as FIFO memory)", pParamName, CONFIG (FifoMaxMem), CONFIG (FifoMaxMem))
    }
 
    return NO_ERROR;
@@ -927,6 +946,9 @@ static APIRET CfgInitializeDlgAcquireFieldNames (void)
 {
    if (CfgLocal.DlgAcquireFieldNames.count() == 0)
    {
+      CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_SPLITFILESWITCH;
+      CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_SPLITFILESIZE;
+      CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_SPLITFILEUNIT;
       CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_EWF_CASENUMBER;
       CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_EWF_EVIDENCENUMBER;
       CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_EWF_EXAMINER;
@@ -936,6 +958,10 @@ static APIRET CfgInitializeDlgAcquireFieldNames (void)
       CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_DEST_IMAGEFILENAME;
       CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_DEST_INFODIRECTORY;
       CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_DEST_INFOFILENAME;
+      CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_HASH_CALC_MD5;
+      CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_HASH_CALC_SHA256;
+      CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_HASH_VERIFY_SRC;
+      CfgLocal.DlgAcquireFieldNames += CFG_DLGACQUIRE_HASH_VERIFY_DST;
    }
    return NO_ERROR;
 }
@@ -953,7 +979,7 @@ static APIRET CfgDestroyDlgAcquireFields (void)
 static APIRET CfgDlgAcquireFieldStart (t_pchar /*pTableId*/, long *pBaseAddr, t_pcchar *ppErrorText)
 {
    CHK (CfgInitializeDlgAcquireFieldNames ())
-   CHK (CfgDestroyDlgAcquireFields())
+//   CHK (CfgDestroyDlgAcquireFields())
    *pBaseAddr = (long) &CfgLocal.CfgBuffDlgAcquireField;
    *ppErrorText = NULL;
 
@@ -964,33 +990,34 @@ static APIRET CfgDlgAcquireFieldSaveAndNext (long *pBaseAddr, t_pcchar *ppErrorT
 {
    t_pCfgBuffDlgAcquireField pBuff;
    t_pCfgDlgAcquireField     pField;
+   bool                       Found=false;
 
    *ppErrorText = NULL;
    pBuff = &CfgLocal.CfgBuffDlgAcquireField;
 
-   if (strcmp(pBuff->FieldName, CFG_DLGACQUIRE_DEST_IMAGEDIRECTORY_ALT) == 0)  // For compatibility. Treat the old, common (image and info)
-      strcpy (pBuff->FieldName, CFG_DLGACQUIRE_DEST_IMAGEDIRECTORY);           // directory name just like the new image directory name
-
    if (CfgLocal.DlgAcquireFieldNames.contains(pBuff->FieldName))
    {
-      for (int i=0; i<CfgLocal.DlgAcquireFields.count(); i++)
+      for (int i=0; (i<CfgLocal.DlgAcquireFields.count()) && !Found; i++)
       {
-         if (CfgLocal.DlgAcquireFields[i]->FieldName == pBuff->FieldName)
-         {
-            *ppErrorText = "This field name figures twice in the table";
-            return NO_ERROR;
-         }
+         pField = CfgLocal.DlgAcquireFields[i];
+         Found = (pField->FieldName == pBuff->FieldName);
       }
-      pField = new (t_CfgDlgAcquireField);
-      pField->FieldName    = pBuff->FieldName;
-      pField->EntryMode    = pBuff->EntryMode;
-      pField->DefaultValue = pBuff->DefaultValue;
-      pField->pLineEdit    = NULL;
-      pField->EwfField = pField->FieldName.startsWith (CFG_DLGACQUIRE_EWF_FIELDID, Qt::CaseInsensitive);
-      pField->DstField = pField->FieldName.startsWith (CFG_DLGACQUIRE_DST_FIELDID, Qt::CaseInsensitive);
-      pField->DirField = pField->FieldName.contains   (CFG_DLGACQUIRE_DIR_FIELDID, Qt::CaseInsensitive);
-
-      CfgLocal.DlgAcquireFields.append (pField);
+      if (!Found)
+      {
+         pField = new (t_CfgDlgAcquireField);
+         pField->FieldName = pBuff->FieldName;
+         pField->EwfField  = pField->FieldName.startsWith (CFG_DLGACQUIRE_EWF_FIELDID , Qt::CaseInsensitive);
+         pField->HashField = pField->FieldName.startsWith (CFG_DLGACQUIRE_HASH_FIELDID, Qt::CaseInsensitive);
+         pField->DstField  = pField->FieldName.startsWith (CFG_DLGACQUIRE_DST_FIELDID , Qt::CaseInsensitive);
+         pField->DirField  = pField->FieldName.contains   (CFG_DLGACQUIRE_DIR_FIELDID , Qt::CaseInsensitive);
+         CfgLocal.DlgAcquireFields.append (pField);
+      }
+      pField->EntryModeImage = pBuff->EntryModeImage;
+      pField->EntryModeClone = pBuff->EntryModeClone;
+      pField->DefaultValue   = pBuff->DefaultValue;
+      pField->pLineEdit      = NULL;
+      pField->pCheckBox      = NULL;
+      pField->pComboBox      = NULL;
    }
    else
    {
