@@ -11,6 +11,23 @@
 //                  window.
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
 #include <float.h>
 
 #include <QString>
@@ -19,6 +36,7 @@
 #include "toolconstants.h"
 
 #include "common.h"
+#include "qtutil.h"
 #include "config.h"
 #include "mainwindow.h"
 #include "main.h"
@@ -40,6 +58,7 @@ void t_Device::Initialise (void)
       CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_DEVICE_BAD_STATE                  ))
       CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_DEVICE_BAD_ABORTREASON            ))
       CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_DEVICE_NOT_CLEAN                  ))
+      CHK_EXIT (TOOL_ERROR_REGISTER_CODE (ERROR_DEVICE_INVALID_MEDIAINFOSTATE     ))
       Initialised = true;
    }
 
@@ -50,16 +69,19 @@ void t_Device::Initialise (void)
    this->SectorSize           = 0;
    this->SectorSizePhys       = 0;
    this->Size                 = 0;
+   this->SpecialDevice        = false;
    this->Removable            = false;
 
    this->State                = Idle;
 //   this->Acquisition.Message       = QString();
    this->AbortRequest         = false;
    this->AbortReason          = None;
+   this->AbortCount           = 0;
    this->DeleteAfterAbort     = false;
+   this->pFifoMemory          = NULL;
    this->pFifoRead            = NULL;
    this->pThreadRead          = NULL;
-   this->pFileSrc             = NULL;
+   this->FileDescSrc          = t_Device::FileDescEmpty;
    this->CurrentReadPos       = 0;
 //   this->BadSectors
 //   this->BadSectorsVerify
@@ -70,16 +92,19 @@ void t_Device::Initialise (void)
 
    this->pThreadWrite         = NULL;
    this->CurrentWritePos      = 0;
-   this->CurrentVerifyPos     = 0;
+   this->CurrentVerifyPosSrc  = 0;
+   this->CurrentVerifyPosDst  = 0;
    this->pFifoWrite           = NULL;
 
    this->pThreadHash          = NULL;
    this->pFifoHashIn          = NULL;
    this->pFifoHashOut         = NULL;
-   memset (&this->MD5Digest         , 0, sizeof(this->MD5Digest         ));
-   memset (&this->MD5DigestVerify   , 0, sizeof(this->MD5DigestVerify   ));
-   memset (&this->SHA256Digest      , 0, sizeof(this->SHA256Digest      ));
-   memset (&this->SHA256DigestVerify, 0, sizeof(this->SHA256DigestVerify));
+   memset (&this->MD5Digest            , 0, sizeof(this->MD5Digest            ));
+   memset (&this->MD5DigestVerifySrc   , 0, sizeof(this->MD5DigestVerifySrc   ));
+   memset (&this->MD5DigestVerifyDst   , 0, sizeof(this->MD5DigestVerifyDst   ));
+   memset (&this->SHA256Digest         , 0, sizeof(this->SHA256Digest         ));
+   memset (&this->SHA256DigestVerifySrc, 0, sizeof(this->SHA256DigestVerifySrc));
+   memset (&this->SHA256DigestVerifyDst, 0, sizeof(this->SHA256DigestVerifyDst));
 
    this->pFifoCompressIn      = NULL;
    this->pFifoCompressOut     = NULL;
@@ -89,9 +114,13 @@ void t_Device::Initialise (void)
 //   this->Acquisition.Path          = QString();
 //   this->Acquisition.ImageFilename = QString();
 //   this->Acquisition.InfoFilename  = QString();
+   this->Acquisition.Clone         = false;
    this->Acquisition.Format        = t_File::NotSet;
-   this->Acquisition.CalcHashes    = false;
-   this->Acquisition.VerifySource  = false;
+   this->Acquisition.CalcMD5       = false;
+   this->Acquisition.CalcSHA256    = false;
+   this->Acquisition.VerifySrc     = false;
+   this->Acquisition.VerifyDst     = false;
+   this->Acquisition.SplitFileSize = 0;
 //   this->Acquisition.CaseNumber    = QString();
 //   this->Acquisition.EvidenceNumber= QString();
 //   this->Acquisition.Examiner      = QString();
@@ -116,8 +145,14 @@ void t_Device::InitialiseDeviceSpecific (const QString &SerialNumber, const QStr
    this->SectorSize     = SectorSize;
    this->SectorSizePhys = SectorSizePhys;
    this->Size           = Size;
-//   this->Size           = std::min (10737418240ULL, Size;  // For faster testing (we make guymager believe that the device is smaller)
-//   this->Size           = std::min (1073741824ULL, Size);  // For faster testing (we make guymager believe that the device is smaller)
+
+   #ifdef FAKE_DEVICE_SIZE
+      // this->Size = GETMIN (10737418240ULL, Size);  // 10 GiB For faster testing (we make guymager believe that the device is smaller)
+      // this->Size = GETMIN (   20485760ULL, Size);  // 20 MiB
+      // this->Size = GETMIN ( 1000000000ULL, Size);  //  1 GB
+      // this->Size = 3ULL*1024ULL*1024ULL*1024ULL*1024ULL;  //  3 TB
+      printf ("\nSize of all devices faked to %llu bytes", this->Size);
+   #endif
 }
 
 t_Device::t_Device (const QString &SerialNumber, const QString &LinuxDevice, const QString &Model,
@@ -137,21 +172,17 @@ t_Device::t_Device (const QString &SerialNumber, const PedDevice *pPedDev)
 
 t_Device::~t_Device()
 {
-   if (pThreadRead   || pFifoRead    ||
-       pThreadHash   || pFifoHashIn  ||
-       pThreadWrite  || pFifoHashOut ||
-       pFileSrc      || pFifoWrite )
+   if (pFifoRead    || pThreadRead  ||
+       pFifoHashIn  || pThreadHash  ||
+       pFifoHashOut || pThreadWrite ||
+       pFifoWrite   || (FileDescSrc != t_Device::FileDescEmpty))
       CHK_EXIT (ERROR_DEVICE_NOT_CLEAN)
-
-   pThreadRead  = NULL; pFifoRead    = NULL;
-   pThreadHash  = NULL; pFifoHashIn  = NULL;
-   pThreadWrite = NULL; pFifoHashOut = NULL;
-   pFileSrc     = NULL; pFifoWrite   = NULL;
 }
 
 bool t_Device::HasHashThread (void) const
 {
-   return Acquisition.CalcHashes && CONFIG (UseSeparateHashThread);
+   return CONFIG (UseSeparateHashThread) && (Acquisition.CalcMD5 ||
+                                             Acquisition.CalcSHA256);
 }
 
 bool t_Device::HasCompressionThreads (void) const
@@ -180,38 +211,81 @@ QVariant t_Device::GetModel (t_pDevice pDevice)
 
 QVariant t_Device::GetState (t_pDevice pDevice)
 {
-   QString State;
+   QString StrState;
 
    switch (pDevice->State)
    {
-      case Idle        : if (pDevice->Local)
-                             State = t_MainWindow::tr("Local device");
-                         else State = t_MainWindow::tr("Idle");
-                         break;
-      case Acquire      : State = t_MainWindow::tr("Acquisition running" ); break;
-      case Verify       : State = t_MainWindow::tr("Verification running"); break;
-      case AcquirePaused: State = t_MainWindow::tr("Device disconnected, acquisition paused" ); break;
-      case VerifyPaused : State = t_MainWindow::tr("Device disconnected, verification paused"); break;
-      case Cleanup      : State = t_MainWindow::tr("Cleanup" ); break;
-      case Finished     : if (pDevice->Acquisition.VerifySource)
+      case Idle         : if (pDevice->Local)
+                                    StrState = t_MainWindow::tr("Local device");
+                          else if (MainGetDeviceList()->UsedAsCloneDestination(pDevice))
+                                    StrState = t_MainWindow::tr("Used in clone operation");
+                               else StrState = t_MainWindow::tr("Idle");
+                          break;
+      case Acquire      : if (pDevice->AbortRequest)
+                               StrState = t_MainWindow::tr("Aborting..."        );
+                          else StrState = t_MainWindow::tr("Acquisition running");
+
+                          if (pDevice->AbortRequest)
                           {
-                             if (HashMD5Match   (&pDevice->MD5Digest   , &pDevice->MD5DigestVerify   ) &&
-                                 HashSHA256Match(&pDevice->SHA256Digest, &pDevice->SHA256DigestVerify))
-                                  State = t_MainWindow::tr("Finished - hashes verified & ok"      );
-                             else State = t_MainWindow::tr("Finished - hash verification mismatch");
+                             switch (pDevice->AbortCount)
+                             {
+                                case  0:
+                                case  1: break;
+                                case  2: StrState += " please be patient"; break;
+                                case  3: StrState += " just keep cool"   ; break;
+                                case  4: StrState += " I said KEEP COOL!"; break;
+                                case  5: StrState += " you start annoying me"; break;
+                                case  6: StrState += " just shut up"; break;
+                                case  7: StrState += " SHUT UP!"; break;
+                                case  8: StrState += " you want a segmentation fault?"; break;
+                                case  9: StrState += " you can have it if you want"; break;
+                                case 10: StrState += " I warn you"; break;
+                                case 11: StrState += " last warning"; break;
+                                case 12: StrState += " ultimate warning"; break;
+                                case 13: StrState += " one more time and I'll seg fault"; break;
+                                case 14: static bool JokeOver = false;
+                                         if (!JokeOver)
+                                         {
+                                            JokeOver = true;
+                                            printf ("\n%c[1;37;41m", ASCII_ESC);  // white on red
+                                            printf ("Signal no. 11 received: Segmentation fault");
+                                            printf ("%c[0m"      , ASCII_ESC);  // standard
+                                            (void) QtUtilSleep (4000);
+                                            printf ("     Just kidding  :-)");
+                                         }
+                                         StrState += " just kidding :-)";
+                                default: break;
+                             }
+                          }
+                          break;
+      case Verify       : StrState = t_MainWindow::tr("Verification running"); break;
+      case AcquirePaused: StrState = t_MainWindow::tr("Device disconnected, acquisition paused" ); break;
+      case VerifyPaused : StrState = t_MainWindow::tr("Device disconnected, verification paused"); break;
+      case Cleanup      : StrState = t_MainWindow::tr("Cleanup" ); break;
+      case Finished     : if ((pDevice->Acquisition.VerifySrc) || (pDevice->Acquisition.VerifyDst))
+                          {
+                             bool Match = true;
+                             if (pDevice->Acquisition.VerifySrc) Match = Match && HashMD5Match   (&pDevice->MD5Digest   , &pDevice->MD5DigestVerifySrc   )
+                                                                               && HashSHA256Match(&pDevice->SHA256Digest, &pDevice->SHA256DigestVerifySrc);
+                             if (pDevice->Acquisition.VerifyDst) Match = Match && HashMD5Match   (&pDevice->MD5Digest   , &pDevice->MD5DigestVerifyDst   )
+                                                                               && HashSHA256Match(&pDevice->SHA256Digest, &pDevice->SHA256DigestVerifyDst);
+                             if (Match)
+                                  StrState = t_MainWindow::tr("Finished - Verified & ok"      );
+                             else StrState = t_MainWindow::tr("Finished - Verification failed");
                           }
                           else
                           {
-                             State = t_MainWindow::tr("Finished");
+                             StrState = t_MainWindow::tr("Finished");
                           }
                           break;
       case Aborted : switch (pDevice->AbortReason)
                      {
-                        case t_Device::None                : State = t_MainWindow::tr("Aborted - Error: Reason is 'none'" ); break;
-                        case t_Device::UserRequest         : State = t_MainWindow::tr("Aborted by user" );                   break;
-                        case t_Device::ThreadWriteFileError: State = t_MainWindow::tr("Aborted - Image file write error" );  break;
-                        case t_Device::ThreadReadFileError : State = t_MainWindow::tr("Aborted - Device read error" );       break;
-                        default                            : CHK_EXIT (ERROR_DEVICE_BAD_ABORTREASON)
+                        case t_Device::None                  : StrState = t_MainWindow::tr("Aborted - Error: Reason is 'none'" ); break;
+                        case t_Device::UserRequest           : StrState = t_MainWindow::tr("Aborted by user" );                   break;
+                        case t_Device::ThreadWriteWriteError : StrState = t_MainWindow::tr("Aborted - Image file write error" );  break;
+                        case t_Device::ThreadWriteVerifyError: StrState = t_MainWindow::tr("Aborted - Image file verify error" ); break;
+                        case t_Device::ThreadReadFileError   : StrState = t_MainWindow::tr("Aborted - Device read error" );       break;
+                        default                              : CHK_EXIT (ERROR_DEVICE_BAD_ABORTREASON)
                      }
                      break;
       default      : CHK_EXIT (ERROR_DEVICE_BAD_STATE)
@@ -221,10 +295,10 @@ QVariant t_Device::GetState (t_pDevice pDevice)
       QString Msg;
       CHK_EXIT (pDevice->GetMessage (Msg))
       if (!Msg.isEmpty())
-         State += " - " + Msg;
+         StrState += " - " + Msg;
    }
 
-   return State;
+   return StrState;
 }
 
 QVariant t_Device::GetSectorSize (t_pDevice pDevice)
@@ -246,6 +320,7 @@ QVariant t_Device::GetSize (t_pDevice pDevice)
 QVariant t_Device::GetSizeHumanFrac (t_pDevice pDevice, bool SI, int FracLen, int UnitThreshold)
 {
    QString      SizeHuman;
+   const char *pBaseUnit = "Byte";
    const char *pUnit;
    double       Sz;
    double       Divisor;
@@ -258,7 +333,7 @@ QVariant t_Device::GetSizeHumanFrac (t_pDevice pDevice, bool SI, int FracLen, in
       UnitThreshold = (int) Divisor;
 
    Sz = pDevice->Size;
-   pUnit = "Byte";
+   pUnit = pBaseUnit;
    if (Sz >= UnitThreshold) { Sz = Sz / Divisor; pUnit = SI ? "kB" : "KiB"; }
    if (Sz >= UnitThreshold) { Sz = Sz / Divisor; pUnit = SI ? "MB" : "MiB"; }
    if (Sz >= UnitThreshold) { Sz = Sz / Divisor; pUnit = SI ? "GB" : "GiB"; }
@@ -270,9 +345,10 @@ QVariant t_Device::GetSizeHumanFrac (t_pDevice pDevice, bool SI, int FracLen, in
 
    if (FracLen == AutoFracLen)
    {
-      if      (Sz >= 100) FracLen = 0;
-      else if (Sz >= 10 ) FracLen = 1;
-      else                FracLen = 2;
+      if (pUnit == pBaseUnit) FracLen = 0; // no frac if unit is bytes
+      else if (Sz >= 100)     FracLen = 0;
+      else if (Sz >= 10 )     FracLen = 1;
+      else                    FracLen = 2;
    }
    SizeHuman = MainGetpNumberLocale()->toString (Sz, 'f', FracLen) + pUnit;
 
@@ -284,6 +360,34 @@ QVariant t_Device::GetSizeHuman (t_pDevice pDevice)
    return GetSizeHumanFrac (pDevice, true, 1);
 }
 
+static QString DeviceMediaInfoStr (t_MediaInfo::t_MediaState State)
+{
+   QString Str;
+   switch (State)
+   {
+      case t_MediaInfo::Unknown: Str = t_MainWindow::tr("Unknown", "Media info string for informing about hidden areas (HPA/DCO)"); break;
+      case t_MediaInfo::No     : Str = t_MainWindow::tr("No"     , "Media info string for informing about hidden areas (HPA/DCO)"); break;
+      case t_MediaInfo::Yes    : Str = t_MainWindow::tr("Yes"    , "Media info string for informing about hidden areas (HPA/DCO)"); break;
+      default: CHK_EXIT (ERROR_DEVICE_INVALID_MEDIAINFOSTATE)
+   }
+   return Str;
+}
+
+QVariant t_Device::GetHiddenAreas (t_pDevice pDevice)
+{
+   QString Str;
+
+   if      ((pDevice->MediaInfo.HasHPA == t_MediaInfo::No     ) && (pDevice->MediaInfo.HasDCO == t_MediaInfo::No     )) Str = t_MainWindow::tr("none"   , "column hidden areas");
+   else if ((pDevice->MediaInfo.HasHPA == t_MediaInfo::Unknown) && (pDevice->MediaInfo.HasDCO == t_MediaInfo::Unknown)) Str = t_MainWindow::tr("unknown", "column hidden areas");
+   else
+   {
+      Str  = "HPA:"    + DeviceMediaInfoStr(pDevice->MediaInfo.HasHPA);
+      Str += " / DCO:" + DeviceMediaInfoStr(pDevice->MediaInfo.HasDCO);
+   }
+
+   return Str;
+}
+
 QVariant t_Device::GetBadSectorCount (t_pDevice pDevice)
 {
    if (!pDevice->StartTimestamp.isNull())
@@ -295,7 +399,8 @@ static void DeviceGetProgress (t_pDevice pDevice, quint64 *pCurrent=NULL, quint6
 {
    quint64 Current, Total;
 
-   if (pDevice->Acquisition.VerifySource)
+   if ((pDevice->Acquisition.VerifySrc) ||
+       (pDevice->Acquisition.VerifyDst))
    {
       Total = 2 * pDevice->Size;
       if (pDevice->StartTimestampVerify.isNull())
@@ -390,8 +495,8 @@ QVariant t_Device::GetRemaining (t_pDevice pDevice)
    int     hh, mm, ss;
    quint64 Current, Total;
 
-   if ((pDevice->State == Acquire) || // Don't display anything if no acquisition is running
-       (pDevice->State == Verify ))
+   if (!pDevice->AbortRequest && ((pDevice->State == Acquire) || // Don't display anything if no acquisition is running
+                                  (pDevice->State == Verify )))
    {
       time (&Now);
       TotalSeconds = pDevice->StartTimestamp.secsTo (QDateTime::currentDateTime());
@@ -403,7 +508,7 @@ QVariant t_Device::GetRemaining (t_pDevice pDevice)
       {
          DeviceGetProgress (pDevice, &Current, &Total);
          ss  = (int) ((double)Total / Current * TotalSeconds); // Estimated total time
-         ss -= TotalSeconds;                                    // Estimated remaining time
+         ss -= TotalSeconds;                                   // Estimated remaining time
          hh = ss / SECONDS_PER_HOUR;   ss %= SECONDS_PER_HOUR;
          mm = ss / SECONDS_PER_MINUTE; ss %= SECONDS_PER_MINUTE;
          snprintf (Buff, sizeof(Buff), "%02d:%02d:%02d", hh, mm, ss);
@@ -428,34 +533,56 @@ QVariant t_Device::GetFifoStatus (t_pDevice pDevice)
    if ((pDevice->State == Acquire) ||  // Don't display anything if no acquisition is running
        (pDevice->State == Verify ))
    {
+      int FifoUsageWrite = 0;                                   // The write thread gets a lot of dummy blocks into its Fifo (from SlotThreadCompressFinished), but 
+                                                                // only 1 of them is taken out (then, the write already exits its loop). If image verification is on
+      if (pDevice->State != Verify)                             // then these dummy blocks remain the in the write fifo, leading to an ugly fifo usage display on the 
+         FifoUsageWrite = DeviceFifoUsage(pDevice->pFifoWrite); // GUI. As I do not like to add code for remove dummy blocks (and waiting until they all have been 
+                                                                // received etc.) I decided to zero them artificially here.
       if (!pDevice->HasHashThread() && !pDevice->HasCompressionThreads())
       {
-         Result = QString ("r %1 w") .arg (DeviceFifoUsage(pDevice->pFifoRead));
+         Result = QString ("r %1 w") .arg (FifoUsageWrite);
       }
       else if (!pDevice->HasHashThread() && pDevice->HasCompressionThreads())
       {
          Result = QString ("r %1 c %2 w") .arg (DeviceFifoUsage(pDevice->pFifoRead      ))
-                                          .arg (DeviceFifoUsage(pDevice->pFifoCompressOut));
+                                          .arg (FifoUsageWrite);
       }
       else if (pDevice->HasHashThread() && !pDevice->HasCompressionThreads())
       {
          Result = QString ("r %1 m %2 w") .arg (DeviceFifoUsage(pDevice->pFifoRead   ))
-                                          .arg (DeviceFifoUsage(pDevice->pFifoHashOut));
+                                          .arg (FifoUsageWrite);
       }
       else if (pDevice->HasHashThread() && pDevice->HasCompressionThreads())
       {
          Result = QString ("r %1 h %2 c %3 w") .arg (DeviceFifoUsage(pDevice->pFifoRead       ))
                                                .arg (DeviceFifoUsage(pDevice->pFifoHashOut    ))
-                                               .arg (DeviceFifoUsage(pDevice->pFifoCompressOut));
+                                               .arg (FifoUsageWrite);
       }
    }
 
    return Result;
 }
 
-
 //lint -restore
 
+const char *t_Device::StateStr (void)
+{
+   const char *pStr;
+   switch (State)
+   {
+      case Idle         : pStr = "Idle";            break;
+      case Acquire      : pStr = "Acquire";         break;
+      case AcquirePaused: pStr = "AcquirePaused";   break;
+      case Verify       : pStr = "Verify";          break;
+      case VerifyPaused : pStr = "VerifyPaused";    break;
+      case Cleanup      : pStr = "Cleanup";         break;
+      case Finished     : pStr = "Finished";        break;
+      case Aborted      : pStr = "Aborte";          break;
+      default           : pStr = "Unknown";
+   }
+
+   return pStr;
+}
 
 // ---------------------------
 //        t_DeviceList
@@ -470,6 +597,7 @@ t_DeviceList::t_DeviceList(void)
    {
       Initialised = true;
       qRegisterMetaType<t_pDeviceList>("t_pDeviceList");
+      qRegisterMetaType<t_pDevice>    ("t_pDevice"    );
    }
 }
 
@@ -502,34 +630,6 @@ t_pDevice t_DeviceList::AppendNew (const QString &SerialNumber, const QString &L
    return pDev;
 }
 
-//t_pDevice t_DeviceList::SearchLinuxDevice  (const QString &LinuxDevice)
-//{
-//   t_pDevice pDev;
-//   int        i;
-//
-//   for (i=0; i<count(); i++)
-//   {
-//      pDev = at(i);
-//      if (pDev->LinuxDevice == LinuxDevice)
-//         return pDev;
-//   }
-//   return NULL;
-//}
-//
-//t_pDevice t_DeviceList::SearchSerialNumber (const QString &SerialNumber)
-//{
-//   t_pDevice pDev;
-//   int        i;
-//
-//   for (i=0; i<count(); i++)
-//   {
-//      pDev = at(i);
-//      if (pDev->SerialNumber == SerialNumber)
-//         return pDev;
-//   }
-//   return NULL;
-//}
-
 APIRET t_DeviceList::MatchDevice (t_pcDevice pDevCmp, t_pDevice &pDevMatch)
 {
    bool Found = false;
@@ -538,15 +638,21 @@ APIRET t_DeviceList::MatchDevice (t_pcDevice pDevCmp, t_pDevice &pDevMatch)
    for (i=0; (i<count()) && !Found; i++)
    {
       pDevMatch = at(i);
-      if (pDevMatch->SerialNumber.isEmpty())
+      if (pDevCmp->SpecialDevice)  // Special device, we may only compare the LinuxDevice string
       {
-         Found = (pDevCmp->SerialNumber.isEmpty() && (pDevMatch->State != t_Device::AcquirePaused) &&
-                                                     (pDevMatch->State != t_Device::VerifyPaused ) &&
-                                                     (pDevMatch->Model == pDevCmp->Model         ) &&
-                                                     (pDevMatch->Size  == pDevCmp->Size          ));
-      }
-      else
+         Found = (pDevMatch->LinuxDevice == pDevCmp->LinuxDevice);
+      }         
+      else  // Not a special device, we can use normal matching
       {
+         if (pDevMatch->SerialNumber.isEmpty())
+         {
+            Found = (pDevCmp->SerialNumber.isEmpty() && (pDevMatch->State != t_Device::AcquirePaused) &&
+                                                        (pDevMatch->State != t_Device::VerifyPaused ) &&
+                                                        (pDevMatch->Model == pDevCmp->Model         ) &&
+                                                        (pDevMatch->Size  == pDevCmp->Size          ));
+         }
+         else
+         {
 //         Found = (pDevMatch->SerialNumber == pDevCmp->SerialNumber);
 //         if (Found)
 //         {                                                                                            // If the serial numbers match, the whole
@@ -554,14 +660,15 @@ APIRET t_DeviceList::MatchDevice (t_pcDevice pDevCmp, t_pDevice &pDevMatch)
 //            if (pDevMatch->Size  != pDevCmp->Size ) return ERROR_DEVICE_SERNR_MATCH_LENGTH_MISMATCH;  // something very strange is going on...
 //         }
 
-// The comparision above cannot be used, because there are certain devices, that behave as if 2 devices aer connected when
-// plugged in - with the same serial number!! Example: I once had a memory stick from Intuix, that reported 2 devices: The
-// memory stick itself and a CD-Rom memory stick itself and a CD-Rom containing device drivers for some strange OS. Both
-// devices had the same serial number. That's why the check now also includes the size and the model.
+// The comparision above cannot be used, because there are certain devices, that behave as if 2 devices are connected when
+// plugged in - with the same serial number!! Example: I once had a memory stick from Intuix that reported 2 devices: The
+// memory stick itself and a CD-Rom containing device drivers for some strange OS. Both devices had the same serial number. 
+// That's why the check now also includes the size and the model.
 
-         Found = ((pDevMatch->SerialNumber == pDevCmp->SerialNumber) &&
-                  (pDevMatch->Size         == pDevCmp->Size )        &&
-                  (pDevMatch->Model        == pDevCmp->Model));
+            Found = ((pDevMatch->SerialNumber == pDevCmp->SerialNumber) &&
+                     (pDevMatch->Size         == pDevCmp->Size )        &&
+                     (pDevMatch->Model        == pDevCmp->Model));
+         }
       }
    }
 
@@ -571,22 +678,21 @@ APIRET t_DeviceList::MatchDevice (t_pcDevice pDevCmp, t_pDevice &pDevMatch)
    return NO_ERROR;
 }
 
-const char *t_Device::StateStr (void)
+bool t_DeviceList::UsedAsCloneDestination (t_pcDevice pDevChk)
 {
-   const char *pStr;
-   switch (State)
-   {
-      case Idle         : pStr = "Idle";            break;
-      case Acquire      : pStr = "Acquire";         break;
-      case AcquirePaused: pStr = "AcquirePaused";   break;
-      case Verify       : pStr = "Verify";          break;
-      case VerifyPaused : pStr = "VerifyPaused";    break;
-      case Cleanup      : pStr = "Cleanup";         break;
-      case Finished     : pStr = "Finished";        break;
-      case Aborted      : pStr = "Aborte";          break;
-      default           : pStr = "Unknown";
-   }
+   t_pcDevice pDevCmp;
+   int         i;
 
-   return pStr;
+   for (i=0; i<count(); i++)
+   {
+      pDevCmp = at(i);
+      if ( (pDevCmp->State != t_Device::Idle    ) &&
+           (pDevCmp->State != t_Device::Finished) &&
+           (pDevCmp->State != t_Device::Aborted ) &&
+           (pDevCmp->Acquisition.Clone          ) &&
+          ((pDevCmp->Acquisition.ImagePath + pDevCmp->Acquisition.ImageFilename) == pDevChk->LinuxDevice))
+         return true;
+   }
+   return false;
 }
 
