@@ -7,12 +7,28 @@
 //                  Section Nouvelles Technologies
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
 #ifndef __DEVICE_H__
 #define __DEVICE_H__
 
 #include "hash.h"
 
-
 #include <parted/parted.h>
 #include <QMetaType>
 #include <QReadWriteLock>
@@ -22,6 +38,7 @@
 #include "fifo.h"
 #include "info.h"
 #include "file.h"
+#include "media.h"
 
 class t_ThreadRead;
 class t_ThreadHash;
@@ -50,28 +67,34 @@ class t_Device
       {
          None,
          UserRequest,
-         ThreadWriteFileError,
+         ThreadWriteWriteError,
+         ThreadWriteVerifyError,
          ThreadReadFileError
       } t_AbortReason;
 
       static const int AutoFracLen       = -1;   // Use up to 2 digits after decimal point (min. 3 significant digits)
       static const int AutoUnitThreshold = -1;
+      static const int FileDescEmpty     = -1;
 
       class t_Acquisition
       {
          public:
-            QString         ImagePath;        // Always end with /
-            QString         InfoPath;         // Always end with /
-            QString         ImageFilename;    // Image filename, without extension
-            QString         InfoFilename;     // Info filename, without extension
-            t_File::Format  Format;
-            bool            CalcHashes;
-            bool            VerifySource;
-            QString         CaseNumber;
-            QString         EvidenceNumber;
-            QString         Examiner;
-            QString         Description;
-            QString         Notes;
+            QString        ImagePath;        // Always end with /
+            QString        InfoPath;         // Always end with /
+            QString        ImageFilename;    // Image filename, without extension
+            QString        InfoFilename;     // Info filename, without extension
+            t_File::Format Format;
+            bool           Clone;
+            bool           CalcMD5;
+            bool           CalcSHA256;
+            bool           VerifySrc;
+            bool           VerifyDst;
+            QString        CaseNumber;
+            QString        EvidenceNumber;
+            QString        Examiner;
+            QString        Description;
+            QString        Notes;
+            quint64        SplitFileSize;    // 0 means do not split
       };
 
    public:
@@ -91,6 +114,7 @@ class t_Device
       static QVariant GetSize                (t_pDevice pDevice);
       static QVariant GetSizeHuman           (t_pDevice pDevice);
       static QVariant GetSizeHumanFrac       (t_pDevice pDevice, bool SI=true, int FracLen=AutoFracLen, int UnitThreshold=AutoUnitThreshold);
+      static QVariant GetHiddenAreas         (t_pDevice pDevice);
       static QVariant GetProgress            (t_pDevice pDevice);
       static QVariant GetCurrentSpeed        (t_pDevice pDevice);
       static QVariant GetAverageSpeed        (t_pDevice pDevice);
@@ -120,27 +144,51 @@ class t_Device
          return Pos;
       }
 
-      inline void SetCurrentVerifyPos (quint64 Pos)
+      inline void SetCurrentVerifyPosSrc (quint64 Pos)
       {
-         SemCurrentVerifyPos.lockForWrite ();
-         CurrentVerifyPos = Pos;
-         SemCurrentVerifyPos.unlock ();
+         SemCurrentVerifyPosSrc.lockForWrite ();
+         CurrentVerifyPosSrc = Pos;
+         SemCurrentVerifyPosSrc.unlock ();
       }
 
-      inline void IncCurrentVerifyPos (int Inc)
+      inline void IncCurrentVerifyPosSrc (int Inc)
       {
-         SemCurrentVerifyPos.lockForWrite ();
-         CurrentVerifyPos += Inc; //lint !e737 Loss of sign
-         SemCurrentVerifyPos.unlock ();
+         SemCurrentVerifyPosSrc.lockForWrite ();
+         CurrentVerifyPosSrc += Inc; //lint !e737 Loss of sign
+         SemCurrentVerifyPosSrc.unlock ();
+      }
+
+      inline void SetCurrentVerifyPosDst (quint64 Pos)
+      {
+         SemCurrentVerifyPosDst.lockForWrite ();
+         CurrentVerifyPosDst = Pos;
+         SemCurrentVerifyPosDst.unlock ();
+      }
+
+      inline void IncCurrentVerifyPosDst (int Inc)
+      {
+         SemCurrentVerifyPosDst.lockForWrite ();
+         CurrentVerifyPosDst += Inc; //lint !e737 Loss of sign
+         SemCurrentVerifyPosDst.unlock ();
       }
 
       inline quint64 GetCurrentVerifyPos (void)
       {
-         quint64 Pos;
-         SemCurrentVerifyPos.lockForRead ();
-         Pos = CurrentVerifyPos;
-         SemCurrentVerifyPos.unlock ();
-         return Pos;
+         quint64 PosSrc;
+         quint64 PosDst;
+
+         SemCurrentVerifyPosSrc.lockForRead ();
+         PosSrc = CurrentVerifyPosSrc;
+         SemCurrentVerifyPosSrc.unlock ();
+
+         SemCurrentVerifyPosDst.lockForRead ();
+         PosDst = CurrentVerifyPosDst;
+         SemCurrentVerifyPosDst.unlock ();
+
+         if      ( Acquisition.VerifySrc &&  Acquisition.VerifyDst) return GETMIN (PosSrc, PosDst);
+         else if ( Acquisition.VerifySrc && !Acquisition.VerifyDst) return PosSrc;
+         else if (!Acquisition.VerifySrc &&  Acquisition.VerifyDst) return PosDst;
+         else                                                       return 0;
       }
 
       inline void AddBadSector (quint64 Sector)
@@ -152,21 +200,21 @@ class t_Device
          SemBadSectors.unlock ();
       }
 
-      APIRET GetBadSectors (QList<quint64> &BadSectorsCopy, bool Verify)
+      APIRET GetBadSectors (QList<quint64> &BadSectorsCopy, bool GetVerify)
       {
          SemBadSectors.lock ();
-         if (Verify)
+         if (GetVerify)
               BadSectorsCopy = BadSectorsVerify;
          else BadSectorsCopy = BadSectors;
          SemBadSectors.unlock ();
          return NO_ERROR;
       }
 
-      quint64 GetBadSectorCount (bool Verify)
+      quint64 GetBadSectorCount (bool GetVerify)
       {
          quint64 Count;
          SemBadSectors.lock ();
-         if (Verify)
+         if (GetVerify)
               Count = BadSectorsVerify.count();  //lint !e732 Loss of sign
          else Count = BadSectors      .count();  //lint !e732 Loss of sign
          SemBadSectors.unlock ();
@@ -214,24 +262,28 @@ class t_Device
             quint64                   SectorSize;
             quint64                   SectorSizePhys;
             quint64                   Size;
+            bool                      SpecialDevice;    // Special device that has been added manually to the device table
             bool                      Removable;
+            t_MediaInfo               MediaInfo;
 
             t_State                   State;
    private: QString                   Message;          // Used by the threads to communicate messages displayed in spreadsheet field "state"
    public:  bool                      AbortRequest;
+            int                       AbortCount;       // Little easter egg  ;-)
             t_AbortReason             AbortReason;
             bool                      DeleteAfterAbort;
 
             t_Acquisition             Acquisition;
             t_Info                    Info;             // Must only be used if Aqcuisition.InfoFilename is valid!
 
-            FILE                    *pFileSrc;
-            quint64                   CurrentReadPos;   // Accessed from different threads, but never at the same time. During acquisition, it is exclusively accessed by the read threads. Using a mutex would be nicer, but could decrease performance.
-   private: quint64                   CurrentVerifyPos; // Accessed concurrently, use appropriate functions
-   private: quint64                   CurrentWritePos;  // Accessed concurrently, use appropriate functions
-   private: QList<quint64>            BadSectors;       // Accessed concurrently, use appropriate functions
-   private: QList<quint64>            BadSectorsVerify; // Accessed concurrently, use appropriate functions
-   public:  bool                      FallbackMode;     // Set if an error occurs while reading a large block. If set, sectors are read individually until the next large block boundary.
+            int                       FileDescSrc;
+            quint64                   CurrentReadPos;      // Accessed from different threads, but never at the same time. During acquisition, it is exclusively accessed by the read threads. Using a mutex would be nicer, but could decrease performance.
+   private: quint64                   CurrentVerifyPosSrc; // Accessed concurrently, use appropriate functions
+   private: quint64                   CurrentVerifyPosDst; // Accessed concurrently, use appropriate functions
+   private: quint64                   CurrentWritePos;     // Accessed concurrently, use appropriate functions
+   private: QList<quint64>            BadSectors;          // Accessed concurrently, use appropriate functions
+   private: QList<quint64>            BadSectorsVerify;    // Accessed concurrently, use appropriate functions
+   public:  bool                      FallbackMode;        // Set if an error occurs while reading a large block. If set, sectors are read individually until the next large block boundary.
             QDateTime                 StartTimestamp;
             QDateTime                 StartTimestampVerify;
             QDateTime                 StopTimestamp;
@@ -241,20 +293,24 @@ class t_Device
             t_ThreadWrite           *pThreadWrite;
             QList<t_ThreadCompress *> ThreadCompressList;
 
-   public:  t_pFifo                  pFifoRead;         // Pointers to the Fifos used by the different
+   public:  t_pFifoMemory            pFifoMemory;
+            t_pFifo                  pFifoRead;         // Pointers to the Fifos used by the different
             t_pFifo                  pFifoHashIn;       // threads. Some of them point to the same Fifos,
             t_pFifo                  pFifoHashOut;      // for instance pFifoRead and pFifoHashIn.
             t_pFifo                  pFifoWrite;
             t_pFifoCompressIn        pFifoCompressIn;
             t_pFifoCompressOut       pFifoCompressOut;
             int                       FifoMaxBlocks;
-            unsigned int              FifoBlockSize;
+            unsigned int              FifoBlockSize;       // The pure size as it would be required by for the data
+            unsigned int              FifoAllocBlockSize;  // The block size as it is used for real allocation
 
 
             t_HashMD5Digest           MD5Digest;
-            t_HashMD5Digest           MD5DigestVerify;
+            t_HashMD5Digest           MD5DigestVerifySrc;
+            t_HashMD5Digest           MD5DigestVerifyDst;
             t_HashSHA256Digest        SHA256Digest;
-            t_HashSHA256Digest        SHA256DigestVerify;
+            t_HashSHA256Digest        SHA256DigestVerifySrc;
+            t_HashSHA256Digest        SHA256DigestVerifyDst;
 
    public:  quint64                   PrevPos;          // Some variables for
             QTime                     PrevTimestamp;    // the current speed
@@ -262,7 +318,8 @@ class t_Device
             bool                      Checked;          // Helper variable for matching algorithm in SlotScanFinished, not used elsewhere.
 
    private: QReadWriteLock            SemCurrentWritePos;
-   private: QReadWriteLock            SemCurrentVerifyPos;
+   private: QReadWriteLock            SemCurrentVerifyPosSrc;
+   private: QReadWriteLock            SemCurrentVerifyPosDst;
    private: QMutex                    SemBadSectors;
    private: QMutex                    SemMessage;
 };
@@ -276,7 +333,9 @@ class t_DeviceList: public QList<t_pDevice>
       t_pDevice AppendNew (const QString &SerialNumber, const PedDevice *pPedDev);
       t_pDevice AppendNew (const QString &SerialNumber, const QString &LinuxDevice, const QString &Model,
                            quint64 SectorSize, quint64 SectorSizePhys, quint64 Size=0);
-      APIRET MatchDevice (t_pcDevice pDevCmp, t_pDevice &pDeviceMatch);
+
+      APIRET MatchDevice            (t_pcDevice pDevCmp, t_pDevice &pDevMatch);
+      bool   UsedAsCloneDestination (t_pcDevice pDevChk);
 };
 
 typedef t_DeviceList *t_pDeviceList;
@@ -294,7 +353,8 @@ enum
    ERROR_DEVICE_SERNR_MATCH_LENGTH_MISMATCH,
    ERROR_DEVICE_BAD_STATE,
    ERROR_DEVICE_BAD_ABORTREASON,
-   ERROR_DEVICE_NOT_CLEAN
+   ERROR_DEVICE_NOT_CLEAN,
+   ERROR_DEVICE_INVALID_MEDIAINFOSTATE
 };
 
 
