@@ -6,7 +6,25 @@
 //                  Service de Police Judiciaire
 //                  Section Nouvelles Technologies
 // ****************************************************************************
+//  Module:         The central queues for pipelined data processing
+// ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
 #ifndef __FIFO_H__
 #define __FIFO_H__
@@ -22,6 +40,17 @@
 #endif
 
 #include "aaff.h"
+#include "aewf.h"
+
+
+
+class   t_FifoMemory;
+typedef t_FifoMemory *t_pFifoMemory;
+
+APIRET FifoMemoryAlloc   (t_pFifoMemory *ppFifoMemory, unsigned int TotalBlocks, unsigned int BlockSize);
+APIRET FifoMemoryFree    (t_pFifoMemory *ppFifoMemory);
+APIRET FifoGetStatistics (quint64 &AllocCalls, quint64 &FreeCalls, qint64 &AllocatedMemory);
+
 
 
 class t_FifoStdLocal;
@@ -44,6 +73,7 @@ typedef struct
    int8_t              EwfWriteCRC;           // For libewf
 
    t_pAaffPreprocess  pAaffPreprocess;
+   t_pAewfPreprocess  pAewfPreprocess;
 
    qulonglong          Nr;                    // Set to FIFO_NR_NOTSET if nr is not known yet
    unsigned char       Buffer[];              //lint !e1501  Has zero size
@@ -58,11 +88,12 @@ class t_Fifo
    public:
       virtual ~t_Fifo  (void) {};
 
-      static  APIRET CreateCompressionOptimised (t_pFifoBlock &pBlock, int CompressionBlockSize);
-      static  APIRET Create                     (t_pFifoBlock &pBlock, unsigned int BufferSize);
-      static  APIRET Destroy                    (t_pFifoBlock &pBlock);
+      static  APIRET Create   (t_pFifoMemory pFifoMemory, t_pFifoBlock &pBlock, unsigned int AllocSize);
+      static  APIRET Destroy  (t_pFifoMemory pFifoMemory, t_pFifoBlock &pBlock);
+      static  void   LogBlock (t_pFifoMemory pFifoMemory, t_pFifoBlock  pBlock, char Marker);
 
-      static  unsigned int GetCompressionOptimisedBufferSize (int CompressionBlockSize);
+      static  unsigned int GetBufferOverhead      (void);
+      static  unsigned int GetOptimisedBufferSize (int BlockSize);
 
       virtual APIRET Insert             (t_pFifoBlock  pBlock) = 0;
       virtual APIRET InsertDummy        (void)                 = 0;
@@ -77,7 +108,7 @@ typedef t_Fifo *t_pFifo;
 class t_FifoStd: public t_Fifo
 {
    public:
-      t_FifoStd (int MaxBlocks=128);
+      t_FifoStd (t_pFifoMemory pFifoMemory, int MaxBlocks=128);
      ~t_FifoStd ();
 
       APIRET Insert             (t_pFifoBlock  pBlock);
@@ -97,7 +128,7 @@ class t_FifoCompressIn: public t_Fifo
 {
    public:
       t_FifoCompressIn (void);
-      t_FifoCompressIn (int SubFifos, int MaxBlocks=128);
+      t_FifoCompressIn (t_pFifoMemory pFifoMemory, int SubFifos, int MaxBlocks=128);
      ~t_FifoCompressIn ();
 
       APIRET Insert             (t_pFifoBlock  pBlock);
@@ -117,7 +148,7 @@ class t_FifoCompressOut: public t_Fifo
 {
    public:
       t_FifoCompressOut (void);
-      t_FifoCompressOut (int SubFifos, int MaxBlocks=128);
+      t_FifoCompressOut (t_pFifoMemory pFifoMemory, int SubFifos, int MaxBlocks=128);
       virtual ~t_FifoCompressOut ();
 
       APIRET Insert             (t_pFifoBlock  pBlock);
@@ -139,9 +170,6 @@ typedef t_FifoCompressIn  *t_pFifoCompressIn;
 typedef t_FifoCompressOut *t_pFifoCompressOut;
 
 
-APIRET FifoGetStatistics (quint64 &AllocCalls, quint64 &FreeCalls, qint64 &AllocatedMemory);
-
-
 // ------------------------------------
 //             Error codes
 // ------------------------------------
@@ -156,7 +184,14 @@ APIRET FifoGetStatistics (quint64 &AllocCalls, quint64 &FreeCalls, qint64 &Alloc
          ERROR_FIFO_START_CORRUPTED,
          ERROR_FIFO_MEMORY_UNDERUN,
          ERROR_FIFO_FUNCTION_NOT_IMPLEMENTED,
-         ERROR_FIFO_CONSTRUCTOR_NOT_SUPPORTED
+         ERROR_FIFO_CONSTRUCTOR_NOT_SUPPORTED,
+         ERROR_FIFO_MEMORY_ALLOC_FAILED,
+         ERROR_FIFO_MEMORY_INVALID_BLOCKSIZE,
+         ERROR_FIFO_MEMORY_ALREADY_ALLOCATED,
+         ERROR_FIFO_MEMORY_NOT_ALLOCATED,
+         ERROR_FIFO_POINTER_TOO_LOW,
+         ERROR_FIFO_POINTER_NOT_ON_BOUNDARY,
+         ERROR_FIFO_POINTER_TOO_HIGH
       };
    #endif
 
