@@ -1,4 +1,6 @@
-<!DOCTYPE TS><TS>
+<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE TS>
+<TS version="2.0">
 <context>
     <name>QObject</name>
     <message>
@@ -36,73 +38,99 @@ Continue anyway?</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>%1 not accessible</source>
-        <comment>Dialog title</comment>
+        <source>Guymager cannot scan the devices connected to this computer.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Guymager is configured to use %1 for scanning the connected devices, but this did not work. Alternatively, you may use %2. Do you want to switch to %2?</source>
+        <source>The selected scan method (&quot;%1&quot;) is not available. Do you want to try another scan method?</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Abort</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Try method &quot;%1&quot;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Creation of a clone</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Linux split dd raw image</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
 <context>
     <name>t_DeviceListModel</name>
-    <message>
-        <source>Serial
-nr.</source>
-        <comment>Column on main screen</comment>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Linux
 device</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
         <source>Model</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>State</source>
-        <comment>Column on main screen</comment>
+        <source>Serial
+nr.</source>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
         <source>Size</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Remarks</source>
+        <comment>Column of device table</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>State</source>
+        <comment>Column of device table</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Hidden
+Areas</source>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
         <source>Bad
 sectors</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
         <source>Progress</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
         <source>Average
 Speed
 [MB/s]</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
         <source>Time
 remaining</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
     <message>
         <source>FIFO queues
 usage
 [%]</source>
-        <comment>Column on main screen</comment>
+        <comment>Column of device table</comment>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -152,10 +180,6 @@ usage
         <source>DestImageDirectory</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>DestDirectory</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>DestImageFilename</source>
         <translation type="unfinished"></translation>
@@ -173,11 +197,6 @@ usage
         <comment>The directory browse button</comment>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Acquisition parameters for %1</source>
-        <comment>Dialog title, %1 is the device (for instance /dev/hdc)</comment>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>File format</source>
         <translation type="unfinished"></translation>
@@ -194,14 +213,6 @@ usage
         <source>Configuration flag WriteToDevNull is set!</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Hash computation</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Re-read source after acquisition for verification (takes twice as long)</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Select destination directory</source>
         <comment>Dialog title</comment>
@@ -214,37 +225,142 @@ usage
     </message>
     <message>
         <source>The filenames contain special characters which are not allowed. Guymager suggests the following changes:
-<byte value="x9"/>Image filename: %1
-<byte value="x9"/>Info filename: %2
+	Image filename: %1
+	Info filename: %2
 Do you accept these changes?</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Images files exist</source>
-        <comment>Dialog title</comment>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>The image files already exist. Do you want to overwrite them?</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Access denied</source>
         <comment>Dialog title</comment>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>HashCalcMD5</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>HashCalcSHA256</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>HashVerifySrc</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>HashVerifyDst</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Hash calculation / verification</source>
+        <translation type="unfinished"></translation>
+    </message>
     <message>
         <source>Guymager cannot write to the directory
-<byte value="x9"/>%1
-This may be due to insufficient access rights. Please choose another directory.</source>
+	%1
+This may be due to insufficient access rights or unsupported filename characters. Please choose another directory.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Calculate MD5</source>
+        <source>Local device, cannot be written</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Calculate SHA-256</source>
+        <source>Too small</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>In use</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Ok for cloning</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The info filename contains special characters which are not allowed. Guymager suggests the following changes:
+	%1
+Do you accept these changes?</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Files exist</source>
+        <comment>Dialog title</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The image or info files already exist. Do you want to overwrite them?</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clone %1</source>
+        <comment>Dialog title, %1 is the device (for instance /dev/hdc)</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Acquire image of %1</source>
+        <comment>Dialog title, %1 is the device (for instance /dev/hdc)</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Device to be cloned</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Used in another clone operation</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>SplitFileSize</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>SplitFileSwitch</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>(file extension %1 or %2)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Incorrect value</source>
+        <comment>Dialog title</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The split file size &quot;%1&quot; is not valid. Only positive numbers can be entered.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The info file already exists. Do you want to overwrite it?</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>SplitFileUnit</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The split size for the selected image format must be in the range %1MiB - %2EiB.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The split size for the configured EWF format must be in the range %1 - %2 MiB.For bigger files, switch to Encase6 format or later (see Guymager configuration file, parameter EwfFormat).</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
+<context>
+    <name>t_DlgAutoExit</name>
+    <message>
+        <source>Abort</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Exit now</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>All acquisitions have ended.
+Guymager will exit automatically in %1 seconds.</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -325,16 +441,6 @@ This may be due to insufficient access rights. Please choose another directory.<
         <source>bytes</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>on</source>
-        <comment>Display that source verification is on</comment>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>off</source>
-        <comment>Display that source verification is off</comment>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Started</source>
         <comment>Start timestamp and running time</comment>
@@ -358,6 +464,20 @@ This may be due to insufficient access rights. Please choose another directory.<
         <comment>Hash calculation is off</comment>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Image verification</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>on</source>
+        <comment>Display that verification is on</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>off</source>
+        <comment>Display that verification is off</comment>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>t_MainWindow</name>
@@ -417,10 +537,6 @@ This may be due to insufficient access rights. Please choose another directory.<
         <source>F5</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Rescan devices and update list</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>&amp;Devices</source>
         <translation type="unfinished"></translation>
@@ -491,19 +607,89 @@ Using libguytools %5</source>
         <source>Finished - Verification failed</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Aborted - Image file verify error</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Unknown</source>
+        <comment>Media info string for informing about hidden areas (HPA/DCO)</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No</source>
+        <comment>Media info string for informing about hidden areas (HPA/DCO)</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Yes</source>
+        <comment>Media info string for informing about hidden areas (HPA/DCO)</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Rescan devices and update table</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Add special device</source>
+        <comment>Menu entry</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Open File</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Invalid device</source>
+        <comment>Dialog title</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Manually added special device</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Device alrady contained</source>
+        <comment>Dialog title</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The selected file or device already is contained in the table.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The device or file cannot be selected because its size is unknown.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>The device or file cannot be selected because it contains 0 bytes.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Used in clone operation</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Aborting...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>none</source>
+        <comment>column hidden areas</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>unknown</source>
+        <comment>column hidden areas</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Exit after acquisitions completed</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>t_Table</name>
-    <message>
-        <source>Acquire</source>
-        <comment>Context menu</comment>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Abort acquisition</source>
-        <comment>Context menu</comment>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Info</source>
         <comment>Context menu</comment>
@@ -558,11 +744,6 @@ Using libguytools %5</source>
         <comment>Info file</comment>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Image</source>
-        <comment>Info file</comment>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Linux device:: %1</source>
         <comment>Info file</comment>
@@ -573,21 +754,6 @@ Using libguytools %5</source>
         <comment>Info file</comment>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Image path and file name:: %1 %2</source>
-        <comment>Info file</comment>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Info  path and file name:: %1 %2</source>
-        <comment>Info file</comment>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Image format:: %1 - file extension is %2</source>
-        <comment>Info file</comment>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Hash calculation:: off</source>
         <comment>Info file</comment>
@@ -678,25 +844,11 @@ Using libguytools %5</source>
         <comment>Info file</comment>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>MD5 hash verified:: %1</source>
-        <comment>Info file</comment>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>SHA256 hash:: %1</source>
         <comment>Info file</comment>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>SHA256 hash verified:: %1</source>
-        <comment>Info file</comment>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>The hash values are identical. The device delivered the same data during acquisition and verification.</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>(ISO format YYYY-MM-DD HH:MM:SS)</source>
         <translation type="unfinished"></translation>
@@ -746,7 +898,124 @@ Using libguytools %5</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>The hash values differ. The device didn&apos;t deliver the same data during acquisition and verification. Maybe you try to acquire the device again. Check as well if the defect sector list was the same during acquisition and verification (see above).</source>
+        <source>Image path and file name:: %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Info  path and file name:: %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Image verification:: on</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Image verification:: off</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Aborted because of image read error during verification</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>MD5 hash verified source:: %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>MD5 hash verified image:: %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>SHA256 hash verified source:: %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>SHA256 hash verified image:: %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Source verification OK. The device delivered the same data during acquisition and verification.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Source verification FAILED. The device didn&apos;t deliver the same data during acquisition and verification. Check if the defect sector list was the same during acquisition and verification (see above).</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Image verification OK. The image contains exactely the data that was written.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Image verification FAILED. The data in the image is different from what was written.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Maybe you try to acquire the device again.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Remove special device</source>
+        <comment>Context menu</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Image meta data</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source> - file extension is %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Acquire image</source>
+        <comment>Context menu</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clone device</source>
+        <comment>Context menu</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Abort</source>
+        <comment>Context menu</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Acquisition</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Format:: %1</source>
+        <comment>Info file</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Device used for cloning - cannot be acquired</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
+<context>
+    <name>t_ThreadScan</name>
+    <message>
+        <source>Cannot scan devices</source>
+        <comment>Dialog title</comment>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>None of the device scan methods worked. Exiting now.</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
