@@ -9,6 +9,23 @@
 //  Module:         Hash wrapper functions for uniform hash interface
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
 #ifndef __HASH_H__
 #define __HASH_H__
 
@@ -47,7 +64,7 @@ typedef struct
 } t_HashSHA256Digest, *t_pHashSHA256Digest;
 
 APIRET HashMD5Init      (t_pHashContextMD5 pContext);
-APIRET HashMD5Append    (t_pHashContextMD5 pContext, void *pData, int DataLen);
+APIRET HashMD5Append    (t_pHashContextMD5 pContext, const void *pData, int DataLen);
 APIRET HashMD5Digest    (t_pHashContextMD5 pContext, t_pHashMD5Digest pDigest);
 APIRET HashMD5DigestStr (t_pHashMD5Digest pDigest, QString &Str);
 bool   HashMD5Match     (t_pHashMD5Digest pDigest1, t_pHashMD5Digest pDigest2);
