@@ -9,6 +9,23 @@
 //  Module:         The info field area
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
 #include <QtGui>
 
 #include "toolconstants.h"
@@ -51,7 +68,8 @@ t_InfoField::t_InfoField (QWidget *pParent)
                                 + "\n" + tr("Current speed")
                                 + "\n" + tr("Started", "Start timestamp and running time")
                                 + "\n" + tr("Hash calculation")
-                                + "\n" + tr("Source verification"));
+                                + "\n" + tr("Source verification")
+                                + "\n" + tr("Image verification"));
 }
 
 t_InfoField::~t_InfoField ()
@@ -91,7 +109,7 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
       }
       else
       {
-         CHK_EXIT (t_File::GetFormatExtension   (pDevice->Acquisition.Format, NULL, &Format))
+         CHK_EXIT (t_File::GetFormatExtension   (pDevice->Acquisition.Format, pDevice->Acquisition.Clone, NULL, &Format))
          StrValue += "\n" + pDevice->Acquisition.ImagePath + pDevice->Acquisition.ImageFilename + QSTR_TO_PSZ(Format);
          StrValue += "\n" + pDevice->Acquisition.InfoPath  + pDevice->Acquisition.InfoFilename  + t_File::pExtensionInfo;
       }
@@ -115,8 +133,10 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
 
          StrValue += "\n" + pDevice->StartTimestamp.toString ("d. MMMM hh:mm:ss");
 
-         if ((pDevice->State == t_Device::Acquire) ||  // Don't display anything if no acquisition is running
-             (pDevice->State == t_Device::Verify ))
+         if ((pDevice->State == t_Device::Acquire      ) ||  // Don't display anything if no acquisition is running
+             (pDevice->State == t_Device::AcquirePaused) ||
+             (pDevice->State == t_Device::Verify       ) ||
+             (pDevice->State == t_Device::VerifyPaused ))
               Seconds = pDevice->StartTimestamp.secsTo (QDateTime::currentDateTime());
          else Seconds = pDevice->StartTimestamp.secsTo (pDevice->StopTimestamp);
          Hours    = Seconds / SECONDS_PER_HOUR  ; Seconds -= Hours   * SECONDS_PER_HOUR;
@@ -127,16 +147,34 @@ void t_InfoField::SlotShowInfo (t_pDevice pDevice)
 
       // Hash
       // ----
-      if (pDevice->Acquisition.CalcHashes)
-           StrValue += "\n" + tr("on" , "Display that hash calculation is on");
-      else StrValue += "\n" + tr("off", "Display that hash calculation is off");
-
+      StrValue += "\n";
+      if (!pDevice->StartTimestamp.isNull())
+      {   
+         if     ((pDevice->Acquisition.CalcMD5) &&
+                 (pDevice->Acquisition.CalcSHA256)) StrValue += tr("MD5 and SHA-256" , "Both hashes are calculated");
+         else if (pDevice->Acquisition.CalcMD5)     StrValue += tr("MD5"    );
+         else if (pDevice->Acquisition.CalcSHA256)  StrValue += tr("SHA-256");
+         else                                       StrValue += tr("off", "Hash calculation is off");
+      }
       // Source verification
       // -------------------
-      if (pDevice->Acquisition.VerifySource)
-           StrValue += "\n" + tr("on" , "Display that source verification is on");
-      else StrValue += "\n" + tr("off", "Display that source verification is off");
-   }
+      StrValue += "\n";
+      if (!pDevice->StartTimestamp.isNull())
+      {   
+         if (pDevice->Acquisition.VerifySrc)
+              StrValue += tr("on" , "Display that verification is on");
+         else StrValue += tr("off", "Display that verification is off");
+      }
+      // Image verification
+      // -------------------
+      StrValue += "\n";
+      if (!pDevice->StartTimestamp.isNull())
+      {   
+         if (pDevice->Acquisition.VerifyDst)
+              StrValue += tr("on" , "Display that verification is on");
+         else StrValue += tr("off", "Display that verification is off");
+      }
+}
    pOwn->pLabelValues->setText (StrValue);
 }
 
