@@ -11,6 +11,24 @@
 //                  deinitialisation.
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
+#include <mcheck.h>
 #include <limits.h>
 #include <signal.h>
 #include <locale.h>
@@ -22,16 +40,10 @@
 
 #include "common.h"
 
-#define MEMWATCH
-#define MEMWATCH_NOCPP
-#include "memwatch.h"
-#undef malloc   // These have been defined in memwatch.h. We must undefine them
-#undef free     // in order to be able to access std::malloc and std::free
-
 #include "toolconstants.h"
-#include "signal/toolsignal.h"
-#include "sysinfo/toolsysinfo.h"
-#include "cfg/toolcfg.h"
+#include "toolsignal.h"
+#include "toolsysinfo.h"
+#include "toolcfg.h"
 
 #include "libewf.h"
 
@@ -41,6 +53,7 @@
 #include "qtutil.h"
 #include "hash.h"
 #include "aaff.h"
+#include "aewf.h"
 #include "mainwindow.h"
 #include "main.h"
 
@@ -49,6 +62,7 @@
 //                Macros
 // ------------------------------------
 
+#define MAIN_ENVVAR_MALLOC_TRACE "MALLOC_TRACE"
 
 // ------------------------------------
 //              Constants
@@ -71,6 +85,9 @@ enum
 #define MAIN_LANGUAGE_FILENAME_QT      "qt_"
 #define MAIN_DEFAULT_MISCFILES_DIR     "/usr/share/guymager/"
 #define MAIN_DEFAULT_LANGUAGE_DIR_QT   "/usr/share/qt4/translations/"
+#ifndef SPLASH_DIR
+   #define SPLASH_DIR                     MAIN_DEFAULT_MISCFILES_DIR
+#endif
 
 // ------------------------------------
 //   Application-wide global variables
@@ -98,11 +115,6 @@ static t_MainLocal MainLocal;
 //       Signal & error handling
 // ------------------------------------
 
-static void MainLogMemMessage (int Ch)
-{
-    printf ("%c", Ch);
-}
-
 static void MainLogQtMessage (QtMsgType MsgType, const char *pMsg)
 {
    const char *pMsgType;
@@ -170,7 +182,7 @@ static APIRET MainWindowAdjustSize (t_MainWindow *pMainWindow)
    if ((MainLocal.pMainWindow->width () > ScreenDx) ||
        (MainLocal.pMainWindow->height() > ScreenDy))
    {
-      LOG_ERROR ("The window manager gave the application a size that's too big: %d x %d (maximum size is %d x %d)",
+      LOG_INFO ("The window manager gave the application a size that's too big: %d x %d (maximum size is %d x %d)",
                   MainLocal.pMainWindow->width (), MainLocal.pMainWindow->height(), ScreenDx, ScreenDy)
 //      CHK_CONST (ERROR_MAIN_INVALID_WINDOW_SIZE)
    }
@@ -178,6 +190,28 @@ static APIRET MainWindowAdjustSize (t_MainWindow *pMainWindow)
    return NO_ERROR;
 }
 
+
+// ------------------------------------
+//          Configuration log
+// ------------------------------------
+
+static void MainCfgLog (t_pcchar pFileName, t_pcchar pFunction, t_int LineNr, t_pcchar pFormat, va_list pArguments)
+{
+   static bool ConsoleLog = false;
+
+   if (!ConsoleLog)
+        ConsoleLog = (strstr (pFormat, "Configuration error"       ) != NULL);
+   else ConsoleLog = (strstr (pFormat, "TOOLCFG_ERROR_CONFIG_ERROR") == NULL);
+
+   if (ConsoleLog)
+   {
+      printf ("\n");
+      vprintf (pFormat, pArguments);
+   }
+
+   t_Log::vEntryInfo (pFileName, pFunction, LineNr, pFormat, pArguments);
+}
+
 // ------------------------------------
 //           Main program
 // ------------------------------------
@@ -188,7 +222,7 @@ APIRET MainPocessEvents (void)
    return NO_ERROR;
 }
 
-static APIRET MainGo (int argc, char *argv[])
+static APIRET MainGo (int argc, char *argv[], bool *pAutoExit)
 {
    QSplashScreen *pSplashScreen = NULL;
    QPixmap       *pSplashPixmap;
@@ -236,12 +270,12 @@ static APIRET MainGo (int argc, char *argv[])
    LOG_INFO ("System: %s", QSTR_TO_PSZ(Uname))
 
    t_Log::GetLibGuyToolsVersion (&pLibGuyToolsVersionInstalled);
-   pLibEwfVersionInstalled = libewf_get_version();
+   pLibEwfVersionInstalled = (const char *) libewf_get_version();
    LOG_INFO ("Libguytools version installed on this PC: %s", pLibGuyToolsVersionInstalled)
    LOG_INFO ("Libewf      version installed on this PC: %s", pLibEwfVersionInstalled)
    meminfo();
    LOG_INFO ("Total amount of memory installed: %lu KB", kb_main_total)
-
+   
 //   pLibEwfVersionCompiled = LIBEWF_VERSION_STRING;
 //
 //   LOG_INFO ("Compiled for libewf version:         %s", pLibEwfVersionCompiled)
@@ -263,7 +297,7 @@ static APIRET MainGo (int argc, char *argv[])
    // --------------
 //   CHK (ToolCfgUseAdjustedCommandLine (MainLocal.pApp->argc(), MainLocal.pApp->argv()))
    CHK (ToolCfgUseAdjustedCommandLine (QApplication::argc(), QApplication::argv()))
-   CHK (ToolCfgSetLogFn (&t_Log::vEntryInfo))
+   CHK (ToolCfgSetLogFn (&MainCfgLog))
    CHK (CfgInit())
    rc = CfgGetCfgFileName (&pCfgFileName, &DefaultUsed);
    if (rc == ERROR_CFG_ONLY_TEMPLATE_GENERATED)
@@ -398,11 +432,6 @@ static APIRET MainGo (int argc, char *argv[])
       }
    }
 
-   // Initialise MemWatch
-   // -------------------
-   if (CONFIG(UseMemWatch))
-      mwSetOutFunc (MainLogMemMessage);
-
    // Initialise signal handler
    // -------------------------
    if (CONFIG (SignalHandling))
@@ -416,6 +445,7 @@ static APIRET MainGo (int argc, char *argv[])
    CHK (QtUtilInit  ())
    CHK (t_File::Init())
    CHK (AaffInit    ())
+   CHK (AewfInit    ())
 
    switch (CONFIG(NumberStyle))
    {
@@ -427,7 +457,7 @@ static APIRET MainGo (int argc, char *argv[])
 
    // Initialise libewf
    // -----------------
-	libewf_set_notify_values (stderr, CONFIG (VerboseLibewf) ? 1 : 0);
+   libewf_set_notify_values (stderr, CONFIG (VerboseLibewf) ? 1 : 0);
 
    // Create central data structures
    // ------------------------------
@@ -437,6 +467,7 @@ static APIRET MainGo (int argc, char *argv[])
    // -----------------------------------------------------------
 
    MainLocal.pMainWindow = new t_MainWindow (MainLocal.pDeviceList);
+   CHK_QT_EXIT (MainLocal.pApp->connect(MainLocal.pMainWindow, SIGNAL(SignalAutoExit(void)), MainLocal.pApp, SLOT(quit(void))))
    CHK (MainWindowAdjustSize (MainLocal.pMainWindow))
 
    if (pSplashScreen)
@@ -451,6 +482,7 @@ static APIRET MainGo (int argc, char *argv[])
    FifoRemaining = FifoAllocs - FifoFrees;
    LOG_INFO ("FIFO buffer statistics: %llu allocated - %llu freed = %lld remaining (%lld bytes)", FifoAllocs, FifoFrees, FifoRemaining, FifoAllocated)
 
+   *pAutoExit = MainLocal.pMainWindow->AutoExit();
 
    // Deinitialise and exit the program
    // ---------------------------------
@@ -461,6 +493,7 @@ static APIRET MainGo (int argc, char *argv[])
    MainLocal.pDeviceList   = NULL;
    MainLocal.pNumberLocale = NULL;
 
+   CHK (AewfDeInit  ())
    CHK (AaffDeInit  ())
    CHK (QtUtilDeInit())
    CHK (HashDeInit  ())
@@ -492,10 +525,19 @@ QLocale *MainGetpNumberLocale (void)
 
 int main (int argc, char *argv[])
 {
-   APIRET rc;
-   int    i;
-   int    Len=0;
+   APIRET        rc;
+   int           i;
+   int           Len=0;
+   int           ExitCode = EXIT_NO_ERROR;
+   bool          AutoExit;
+   const char  *pFilenameMallocTracing;
 
+   pFilenameMallocTracing = getenv (MAIN_ENVVAR_MALLOC_TRACE);
+   if (pFilenameMallocTracing != NULL)  // Switch on memory allocation tracing,
+   {
+      printf ("\nSwitching malloc tracing on, output file is %s", pFilenameMallocTracing);
+      mtrace();                                    // see tracing functionality in GNU C lib
+   }
    setbuf(stdout, NULL);
    setbuf(stderr, NULL);
    setlocale (LC_ALL, "");
@@ -517,7 +559,9 @@ int main (int argc, char *argv[])
       strcat (MainLocal.pCommandLine, argv[i]);
    }
 
-   rc = MainGo (argc, argv);
+   rc = MainGo (argc, argv, &AutoExit);
+   if (AutoExit)
+      ExitCode = EXIT_AUTOEXIT;
 
    free (MainLocal.pCommandLine);
 
@@ -526,6 +570,11 @@ int main (int argc, char *argv[])
         printf ("Error in configuration file. See log file for details.\n");
    else CHK_EXIT (rc)
 
-   return rc;
+   return ExitCode;
+}
+
+t_pDeviceList MainGetDeviceList (void)
+{
+   return MainLocal.pDeviceList;
 }
 
