@@ -10,6 +10,22 @@
 //                  widget
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
 #include <QtGui>
 
@@ -27,6 +43,7 @@
 #include "infofield.h"
 #include "table.h"
 #include "dlgmessage.h"
+#include "dlgautoexit.h"
 
 
 // -----------------------------
@@ -35,7 +52,6 @@
 
 const int MAINWINDOW_WAIT_FOR_THREADS_TO_ABORT = 5000;
 
-
 // -----------------------------
 //  Table model for device list
 // -----------------------------
@@ -63,19 +79,20 @@ t_DeviceListModel::t_DeviceListModel (t_pDeviceList pDeviceList)
    #define RIGHT  (Qt::AlignRight   | Qt::AlignVCenter)
    #define CENTER (Qt::AlignHCenter | Qt::AlignVCenter)
 
-      COL_ASSOC (tr("Serial\nnr."            , "Column on main screen"), t_Device::GetSerialNumber  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-      COL_ASSOC (tr("Linux\ndevice"          , "Column on main screen"), t_Device::GetLinuxDevice   , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-      COL_ASSOC (tr("Model"                  , "Column on main screen"), t_Device::GetModel         , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-      COL_ASSOC (tr("State"                  , "Column on main screen"), t_Device::GetState         , t_ItemDelegate::DISPLAYTYPE_STATE   , LEFT  , 200)
-      COL_ASSOC (tr("Size"                   , "Column on main screen"), t_Device::GetSizeHuman     , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
-      COL_ASSOC (tr("Bad\nsectors"           , "Column on main screen"), t_Device::GetBadSectorCount, t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
-      COL_ASSOC (tr("Progress"               , "Column on main screen"), t_Device::GetProgress      , t_ItemDelegate::DISPLAYTYPE_PROGRESS, LEFT  ,   0)
-      COL_ASSOC (tr("Average\nSpeed\n[MB/s]" , "Column on main screen"), t_Device::GetAverageSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
-      COL_ASSOC (tr("Time\nremaining"        , "Column on main screen"), t_Device::GetRemaining     , t_ItemDelegate::DISPLAYTYPE_STANDARD, CENTER,   0)
-      COL_ASSOC (tr("FIFO queues\nusage\n[%]", "Column on main screen"), t_Device::GetFifoStatus    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
-//      COL_ASSOC (tr("Sector\nsize\nlog."    , "Column on main screen"), t_Device::GetSectorSize    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
-//      COL_ASSOC (tr("Sector\nsize\nphys."   , "Column on main screen"), t_Device::GetSectorSizePhys, t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
-//      COL_ASSOC (tr("Current\nSpeed\n[MB/s]", "Column on main screen"), t_Device::GetCurrentSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
+      COL_ASSOC (tr("Serial\nnr."            , "Column of device table"), t_Device::GetSerialNumber  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
+      COL_ASSOC (tr("Linux\ndevice"          , "Column of device table"), t_Device::GetLinuxDevice   , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
+      COL_ASSOC (tr("Model"                  , "Column of device table"), t_Device::GetModel         , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
+      COL_ASSOC (tr("State"                  , "Column of device table"), t_Device::GetState         , t_ItemDelegate::DISPLAYTYPE_STATE   , LEFT  , 200)
+      COL_ASSOC (tr("Size"                   , "Column of device table"), t_Device::GetSizeHuman     , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
+      COL_ASSOC (tr("Hidden\nAreas"          , "Column of device table"), t_Device::GetHiddenAreas   , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
+      COL_ASSOC (tr("Bad\nsectors"           , "Column of device table"), t_Device::GetBadSectorCount, t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
+      COL_ASSOC (tr("Progress"               , "Column of device table"), t_Device::GetProgress      , t_ItemDelegate::DISPLAYTYPE_PROGRESS, LEFT  ,   0)
+      COL_ASSOC (tr("Average\nSpeed\n[MB/s]" , "Column of device table"), t_Device::GetAverageSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, RIGHT ,   0)
+      COL_ASSOC (tr("Time\nremaining"        , "Column of device table"), t_Device::GetRemaining     , t_ItemDelegate::DISPLAYTYPE_STANDARD, CENTER,   0)
+      COL_ASSOC (tr("FIFO queues\nusage\n[%]", "Column of device table"), t_Device::GetFifoStatus    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT  ,   0)
+//    COL_ASSOC (tr("Sector\nsize\nlog."     , "Column of device table"), t_Device::GetSectorSize    , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
+//    COL_ASSOC (tr("Sector\nsize\nphys."    , "Column of device table"), t_Device::GetSectorSizePhys, t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
+//    COL_ASSOC (tr("Current\nSpeed\n[MB/s]" , "Column of device table"), t_Device::GetCurrentSpeed  , t_ItemDelegate::DISPLAYTYPE_STANDARD, LEFT, 0      )
    #undef COL_ASSOC
    #undef LEFT
    #undef RIGHT
@@ -184,8 +201,11 @@ class t_MainWindowLocal
       t_pInfoField           pInfoField;
       t_ThreadScan          *pThreadScan;
       QAction               *pActionRescan;
+      QAction               *pActionAutoExit;
       t_ThreadScanWorker    *pScanWorker;
       QTimer                *pTimerRefresh;
+      bool                    AutoExit;
+      bool                    AllAbortedByUser;
 };
 
 static APIRET MainWindowRegisterErrorCodes (void)
@@ -193,6 +213,7 @@ static APIRET MainWindowRegisterErrorCodes (void)
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_CONSTRUCTOR_NOT_SUPPORTED))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_INVALID_COLUMN))
    CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_INVALID_DATATYPE))
+   CHK (TOOL_ERROR_REGISTER_CODE (ERROR_MAINWINDOW_DEVICE_NOT_FOUND))
 
    return NO_ERROR;
 }
@@ -207,14 +228,18 @@ APIRET t_MainWindow::CreateMenu (void)
    // -------
    pOwn->pActionRescan = new QAction (tr("&Rescan"), this);
    pOwn->pActionRescan->setShortcut  (tr("F5"));
-   pOwn->pActionRescan->setToolTip   (tr("Rescan devices and update list"));
+   pOwn->pActionRescan->setToolTip   (tr("Rescan devices and update table"));
 
    // Menu
    // ----
    pMenu = pMenuBar->addMenu (tr("&Devices"));
    pMenu->addAction (pOwn->pActionRescan);
+   pMenu->addAction (tr("Add special device", "Menu entry"), this, SLOT(SlotAddSpecialDevice()));
 
    pMenu = pMenuBar->addMenu (tr("&Misc", "Menu entry"));
+   pOwn->pActionAutoExit = pMenu->addAction (tr("Exit after acquisitions completed"));
+   pOwn->pActionAutoExit->setCheckable(true);
+   pOwn->pActionAutoExit->setChecked ((CONFIG(AutoExit)!=0));
    pMenu->addAction (tr("Debug", "Menu entry"), this, SLOT(SlotDebug()));
 
    pMenu = pMenuBar->addMenu (tr("&Help", "Menu entry"));
@@ -249,6 +274,8 @@ t_MainWindow::t_MainWindow (t_pDeviceList pDeviceList, QWidget *pParent, Qt::WFl
 
    pOwn = new t_MainWindowLocal;
    pOwn->pDeviceList = pDeviceList;
+   pOwn->AutoExit         = false;
+   pOwn->AllAbortedByUser = false;
 
    CHK_EXIT (CreateMenu ())
    setWindowTitle ("GUYMAGER");
@@ -262,7 +289,7 @@ t_MainWindow::t_MainWindow (t_pDeviceList pDeviceList, QWidget *pParent, Qt::WFl
    pOwn->pCentralWidget = new QWidget (this);
    QVBoxLayout *pLayout = new QVBoxLayout (pOwn->pCentralWidget);
 
-   pOwn->pTable                      = new t_Table (pOwn->pCentralWidget, pDeviceList);
+   pOwn->pTable                      = new t_Table (pOwn->pCentralWidget, this, pDeviceList);
    pOwn->pDeviceListModel            = new t_DeviceListModel     (pDeviceList);
    pOwn->pProxyModel                 = new QSortFilterProxyModel (pOwn->pCentralWidget);
    pOwn->pProxyModel->setSourceModel (pOwn->pDeviceListModel);
@@ -273,8 +300,9 @@ t_MainWindow::t_MainWindow (t_pDeviceList pDeviceList, QWidget *pParent, Qt::WFl
    pLayout->addWidget (pOwn->pTable);
    pLayout->addWidget (pOwn->pInfoField);
 
-   CHK_QT_EXIT (connect (pOwn->pTable->horizontalHeader(), SIGNAL(sectionClicked      (int      )), pOwn->pTable    , SLOT(sortByColumn  (int      ))))
-   CHK_QT_EXIT (connect (pOwn->pTable                    , SIGNAL(SignalDeviceSelected(t_pDevice)), pOwn->pInfoField, SLOT (SlotShowInfo (t_pDevice))))
+   CHK_QT_EXIT (connect (pOwn->pTable->horizontalHeader(), SIGNAL(sectionClicked            (int      )), pOwn->pTable    , SLOT(sortByColumn  (int      ))))
+   CHK_QT_EXIT (connect (pOwn->pTable                    , SIGNAL(SignalDeviceSelected      (t_pDevice)), pOwn->pInfoField, SLOT (SlotShowInfo (t_pDevice))))
+   CHK_QT_EXIT (connect (pOwn->pTable                    , SIGNAL(SignalAllAcquisitionsEnded(void     )), this            , SLOT (SlotAutoExit (void     ))))
 
    setCentralWidget (pOwn->pCentralWidget);
 
@@ -305,6 +333,80 @@ void t_MainWindow::SlotRefresh (void)
    pOwn->pInfoField->SlotShowInfo (pDevice);
 }
 
+
+void t_MainWindow::SlotAddSpecialDevice (void)
+{
+   static QDir  LastDir("/");
+   QString      FileName;
+   QFileInfo    FileInfo ;
+   t_pDevice   pDevice;
+   t_pDevice   pDeviceFound;
+   FILE       *pFile = NULL;
+   bool         Error;
+   qint64       Size;
+
+   for (;;)
+   {
+      FileName = QFileDialog::getOpenFileName (this, tr("Open File"), LastDir.canonicalPath ());
+      FileInfo.setFile (FileName);
+      LastDir = FileInfo.dir();
+      if (FileName.isNull())
+         return;
+      Size = FileInfo.size();
+      if (Size <= 0)
+      {
+         LOG_INFO ("Qt thinks the file size for %s is %lld, trying seek to get it", QSTR_TO_PSZ(FileName), Size)
+         pFile = fopen64 (QSTR_TO_PSZ(FileName), "r");
+         Error = (pFile == NULL);
+         if (!Error)
+            Error = (fseeko64 (pFile, 0, SEEK_END) != 0);
+         if (!Error)
+         {
+            Size  = ftello64 (pFile);
+            Error = (Size < 0);
+         }
+
+         if (Error)
+            QMessageBox::information (this, tr ("Invalid device", "Dialog title"), tr("The device or file cannot be selected because its size is unknown."), QMessageBox::Ok);
+         else if (Size <= 0)
+            QMessageBox::information (this, tr ("Invalid device", "Dialog title"), tr("The device or file cannot be selected because it contains 0 bytes."), QMessageBox::Ok);
+         if (Size <= 0)
+            continue;
+      }
+      pDevice = new t_Device ("", FileName, tr("Manually added special device"), 512, 512, Size);
+      pDevice->SpecialDevice = true;
+      CHK_EXIT (pOwn->pDeviceList->MatchDevice (pDevice, pDeviceFound))
+      if (pDeviceFound)
+      {
+         delete pDevice;
+         QMessageBox::information (this, tr ("Device alrady contained", "Dialog title"), tr("The selected file or device already is contained in the table."), QMessageBox::Ok);
+         continue;
+      }
+      break;
+   }
+   LOG_INFO ("Adding special device %s with size %lld", QSTR_TO_PSZ(FileName), Size)
+
+   pOwn->pDeviceList->append (pDevice);
+   pOwn->pDeviceListModel->SlotRefresh();
+   pOwn->pTable->resizeColumnsToContents();
+}
+
+APIRET t_MainWindow::RemoveSpecialDevice (t_pDevice pDevice)
+{
+   int i;
+
+   i = pOwn->pDeviceList->indexOf (pDevice);
+   if (i < 0)
+      return ERROR_MAINWINDOW_DEVICE_NOT_FOUND;
+   pOwn->pDeviceList->removeAt(i);
+
+   pOwn->pDeviceListModel->SlotRefresh();
+   pOwn->pTable->resizeColumnsToContents();
+
+   return NO_ERROR;
+}
+
+
 // SlotScanFinished provides us with the new device list. Now, our current device list needs to be updated. The rules are:
 //  (1) For devices, who are in both lists:
 //        Set the state of the device in the current list to Connected. Thus, devices that were temporarly disconnected
@@ -324,18 +426,30 @@ void t_MainWindow::SlotScanFinished (t_pDeviceList pNewDeviceList)
    t_pDevice     pDev, pNewDev;
    int            i;
 
+//   LOG_INFO ("%d devices found", pNewDeviceList->count())
    pDeviceList = pOwn->pDeviceList;
    for (i=0; i<pDeviceList->count(); i++)
    {
       pDev = pDeviceList->at (i);
-      pDev->Checked = false;  // Checked is used to remember which devices we have seen and which ones not
+      if (pDev->SpecialDevice)
+           pDev->Checked = true;   // Checked is used to remember which devices we have seen and which ones not. Treat
+      else pDev->Checked = false;  // special devices as already seen, thus, they are going to be removed from the list.
    }
 
    for (i=0; i<pNewDeviceList->count(); i++)
    {
       pNewDev = pNewDeviceList->at (i);
       CHK_EXIT (pDeviceList->MatchDevice (pNewDev, pDev))
-//      LOG_INFO ("Result of match for %s: %s", QSTR_TO_PSZ (pNewDev->LinuxDevice), pDev ? "Match" : "NoMatch")
+//      if (pDev)
+//      {
+//         LOG_INFO ("device %s %s %s %llu matches %s %s %s %llu", 
+//            QSTR_TO_PSZ (pNewDev->LinuxDevice), QSTR_TO_PSZ (pNewDev->Model), QSTR_TO_PSZ (pNewDev->SerialNumber), pNewDev->Size,
+//            QSTR_TO_PSZ (   pDev->LinuxDevice), QSTR_TO_PSZ (   pDev->Model), QSTR_TO_PSZ (   pDev->SerialNumber), pNewDev->Size)
+//      }
+//      else
+//      {
+//         LOG_INFO ("No match for device %s %s %s %llu", QSTR_TO_PSZ (pNewDev->LinuxDevice), QSTR_TO_PSZ (pNewDev->Model), QSTR_TO_PSZ (pNewDev->SerialNumber), pNewDev->Size)
+//      }
 
       if (pDev)
       {
@@ -406,6 +520,7 @@ void t_MainWindow::closeEvent (QCloseEvent *pEvent)
                                       QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
       if (Button == QMessageBox::Yes)
       {  // stop acquisitions
+         pOwn->AllAbortedByUser = true;
          LOG_INFO ("User confirms abortion in order to quit")
          CHK_EXIT (pOwn->pTable->AbortAllAcquisitions ())
          CHK_EXIT (QtUtilSleep (MAINWINDOW_WAIT_FOR_THREADS_TO_ABORT))
@@ -418,6 +533,7 @@ void t_MainWindow::closeEvent (QCloseEvent *pEvent)
    }
 }
 
+
 void t_MainWindow::SlotDebug (void)
 {
    QString DebugInfo;
@@ -443,7 +559,7 @@ void t_MainWindow::SlotAboutGuymager (void)
 
    t_Log::GetLibGuyToolsVersion (&pLibGuyToolsVersionInstalled);
 
-   pLibEwfVersionInstalled = libewf_get_version ();
+   pLibEwfVersionInstalled = (const char *) libewf_get_version ();
 
    QMessageBox::about(this, tr("About GUYMAGER", "Dialog title"),
                             tr("GUYMAGER is a Linux-based forensic imaging tool\n\n"
@@ -464,6 +580,39 @@ void t_MainWindow::SlotAboutQt (void)
    QMessageBox::aboutQt (this, tr("About Qt", "Dialog title"));
 }
 
+void t_MainWindow::SlotAutoExit (void)
+{
+   bool AutoExit;
+
+   if (!pOwn->AllAbortedByUser)
+   {
+      if (pOwn->pActionAutoExit->isChecked())
+      {
+         LOG_INFO ("Showing autoexit countdown dialog (%d seconds)", CONFIG(AutoExitCountdown))
+         CHK_EXIT (t_DlgAutoExit::Show (&AutoExit))
+         if (AutoExit)
+         {
+            LOG_INFO ("Autoexit is becoming active, Guymager is exiting automatically now.")
+            pOwn->AutoExit = true;
+            emit SignalAutoExit();
+         }
+         else
+         {
+            LOG_INFO ("Autoexit coutdown has been stopped. Guymager will continue to run.")
+         }
+      }
+      else
+      {
+         LOG_INFO ("Signal \"all acquisitions ended\" received, but autoexit flag is off, doing nothing.")
+      }
+   }
+}
+
+bool t_MainWindow::AutoExit (void)
+{
+   return pOwn->AutoExit; 
+}
+
 t_MainWindow::~t_MainWindow ()
 {
    CHK_EXIT (pOwn->pThreadScan->Stop ())
