@@ -7,9 +7,9 @@
 # or similar instead
 
 if [ -e ../../scripts/package.sh ]; then
-   ../../scripts/package.sh guymager
+   ../../scripts/package.sh guymager-beta
 else
-   ../../../scripts/package.sh guymager # when called from the branches or tags directories
+   ../../../scripts/package.sh guymager-beta # when called from the branches or tags directories
 fi
 
 ./compileinfo.sh | grep Version | awk -F\" ' { print "Version "$2 } '
