@@ -1,11 +1,9 @@
-// Guy: This file has been downloaded from www.spale.com/download/scrypt/scrypt1.0/sha256.c. It
-// has been optimised for speedon i386 and amd64 architecture and corrected for powerpc.
-
 // sha256 calculation on i386 and amd64 processors.
 /*
  *  FIPS-180-2 compliant SHA-256 implementation
  *
  *  Copyright (C) 2001-2003  Christophe Devine
+ *  Copyright (C) 2010,2011  Guy Voncken <vogu00@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +20,16 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+// This file has initially been downloaded from
+//     www.spale.com/download/scrypt/scrypt1.0/sha256.c
+//
+// The following optimisation and debugging work has been done by Guy Voncken:
+//     - The CPU intensive parts have been rewritten in assembler for better
+//       performance on i386 and amd64 machines.
+//     - The code has been corrected for big endian machines. The corrections
+//       have been tested on a PowerPC machine.
+
+
 #include <string.h>
 #include <netinet/in.h>
 
@@ -32,10 +40,10 @@
 #endif
 
 
-// Guy: The original GET_UINT32 and PUT_UINT32 are
+// Guy: The original GET_UINT32 and PUT_UINT32 macros are
 //   - too slow
 //   - not working on powerpc
-// They have been replaced by the powerful htonl/ntohl functions.
+// I replaced them by the powerful htonl/ntohl functions.
 
 /*
 #define GET_UINT32(n,b,i)                       \
@@ -389,70 +397,70 @@ void sha256_process( sha256_context *ctx, uint8 data[64] )
     G = ctx->state[6];
     H = ctx->state[7];
 
-    P( A, B, C, D, E, F, G, H, W[ 0], 0x428A2F98 );
-    P( H, A, B, C, D, E, F, G, W[ 1], 0x71374491 );
-    P( G, H, A, B, C, D, E, F, W[ 2], 0xB5C0FBCF );
-    P( F, G, H, A, B, C, D, E, W[ 3], 0xE9B5DBA5 );
-    P( E, F, G, H, A, B, C, D, W[ 4], 0x3956C25B );
-    P( D, E, F, G, H, A, B, C, W[ 5], 0x59F111F1 );
-    P( C, D, E, F, G, H, A, B, W[ 6], 0x923F82A4 );
-    P( B, C, D, E, F, G, H, A, W[ 7], 0xAB1C5ED5 );
-    P( A, B, C, D, E, F, G, H, W[ 8], 0xD807AA98 );
-    P( H, A, B, C, D, E, F, G, W[ 9], 0x12835B01 );
-    P( G, H, A, B, C, D, E, F, W[10], 0x243185BE );
-    P( F, G, H, A, B, C, D, E, W[11], 0x550C7DC3 );
-    P( E, F, G, H, A, B, C, D, W[12], 0x72BE5D74 );
-    P( D, E, F, G, H, A, B, C, W[13], 0x80DEB1FE );
-    P( C, D, E, F, G, H, A, B, W[14], 0x9BDC06A7 );
-    P( B, C, D, E, F, G, H, A, W[15], 0xC19BF174 );
-    PR(A, B, C, D, E, F, G, H,   16 , 0xE49B69C1 );
-    PR(H, A, B, C, D, E, F, G,   17 , 0xEFBE4786 );
-    PR(G, H, A, B, C, D, E, F,   18 , 0x0FC19DC6 );
-    PR(F, G, H, A, B, C, D, E,   19 , 0x240CA1CC );
-    PR(E, F, G, H, A, B, C, D,   20 , 0x2DE92C6F );
-    PR(D, E, F, G, H, A, B, C,   21 , 0x4A7484AA );
-    PR(C, D, E, F, G, H, A, B,   22 , 0x5CB0A9DC );
-    PR(B, C, D, E, F, G, H, A,   23 , 0x76F988DA );
-    PR(A, B, C, D, E, F, G, H,   24 , 0x983E5152 );
-    PR(H, A, B, C, D, E, F, G,   25 , 0xA831C66D );
-    PR(G, H, A, B, C, D, E, F,   26 , 0xB00327C8 );
-    PR(F, G, H, A, B, C, D, E,   27 , 0xBF597FC7 );
-    PR(E, F, G, H, A, B, C, D,   28 , 0xC6E00BF3 );
-    PR(D, E, F, G, H, A, B, C,   29 , 0xD5A79147 );
-    PR(C, D, E, F, G, H, A, B,   30 , 0x06CA6351 );
-    PR(B, C, D, E, F, G, H, A,   31 , 0x14292967 );
-    PR(A, B, C, D, E, F, G, H,   32 , 0x27B70A85 );
-    PR(H, A, B, C, D, E, F, G,   33 , 0x2E1B2138 );
-    PR(G, H, A, B, C, D, E, F,   34 , 0x4D2C6DFC );
-    PR(F, G, H, A, B, C, D, E,   35 , 0x53380D13 );
-    PR(E, F, G, H, A, B, C, D,   36 , 0x650A7354 );
-    PR(D, E, F, G, H, A, B, C,   37 , 0x766A0ABB );
-    PR(C, D, E, F, G, H, A, B,   38 , 0x81C2C92E );
-    PR(B, C, D, E, F, G, H, A,   39 , 0x92722C85 );
-    PR(A, B, C, D, E, F, G, H,   40 , 0xA2BFE8A1 );
-    PR(H, A, B, C, D, E, F, G,   41 , 0xA81A664B );
-    PR(G, H, A, B, C, D, E, F,   42 , 0xC24B8B70 );
-    PR(F, G, H, A, B, C, D, E,   43 , 0xC76C51A3 );
-    PR(E, F, G, H, A, B, C, D,   44 , 0xD192E819 );
-    PR(D, E, F, G, H, A, B, C,   45 , 0xD6990624 );
-    PR(C, D, E, F, G, H, A, B,   46 , 0xF40E3585 );
-    PR(B, C, D, E, F, G, H, A,   47 , 0x106AA070 );
-    PR(A, B, C, D, E, F, G, H,   48 , 0x19A4C116 );
-    PR(H, A, B, C, D, E, F, G,   49 , 0x1E376C08 );
-    PR(G, H, A, B, C, D, E, F,   50 , 0x2748774C );
-    PR(F, G, H, A, B, C, D, E,   51 , 0x34B0BCB5 );
-    PR(E, F, G, H, A, B, C, D,   52 , 0x391C0CB3 );
-    PR(D, E, F, G, H, A, B, C,   53 , 0x4ED8AA4A );
-    PR(C, D, E, F, G, H, A, B,   54 , 0x5B9CCA4F );
-    PR(B, C, D, E, F, G, H, A,   55 , 0x682E6FF3 );
-    PR(A, B, C, D, E, F, G, H,   56 , 0x748F82EE );
-    PR(H, A, B, C, D, E, F, G,   57 , 0x78A5636F );
-    PR(G, H, A, B, C, D, E, F,   58 , 0x84C87814 );
-    PR(F, G, H, A, B, C, D, E,   59 , 0x8CC70208 );
-    PR(E, F, G, H, A, B, C, D,   60 , 0x90BEFFFA );
-    PR(D, E, F, G, H, A, B, C,   61 , 0xA4506CEB );
-    PR(C, D, E, F, G, H, A, B,   62 , 0xBEF9A3F7 );
-    PR(B, C, D, E, F, G, H, A,   63 , 0xC67178F2 );
+    P( A, B, C, D, E, F, G, H, W[ 0],  0x428A2F98 );
+    P( H, A, B, C, D, E, F, G, W[ 1],  0x71374491 );
+    P( G, H, A, B, C, D, E, F, W[ 2], -0x4A3F0431 );
+    P( F, G, H, A, B, C, D, E, W[ 3], -0x164A245B );
+    P( E, F, G, H, A, B, C, D, W[ 4],  0x3956C25B );
+    P( D, E, F, G, H, A, B, C, W[ 5],  0x59F111F1 );
+    P( C, D, E, F, G, H, A, B, W[ 6], -0x6DC07D5C );
+    P( B, C, D, E, F, G, H, A, W[ 7], -0x54E3A12B );
+    P( A, B, C, D, E, F, G, H, W[ 8], -0x27F85568 );
+    P( H, A, B, C, D, E, F, G, W[ 9],  0x12835B01 );
+    P( G, H, A, B, C, D, E, F, W[10],  0x243185BE );
+    P( F, G, H, A, B, C, D, E, W[11],  0x550C7DC3 );
+    P( E, F, G, H, A, B, C, D, W[12],  0x72BE5D74 );
+    P( D, E, F, G, H, A, B, C, W[13], -0x7F214E02 );
+    P( C, D, E, F, G, H, A, B, W[14], -0x6423F959 );
+    P( B, C, D, E, F, G, H, A, W[15], -0x3E640E8C );
+    PR(A, B, C, D, E, F, G, H,   16 , -0x1B64963F );
+    PR(H, A, B, C, D, E, F, G,   17 , -0x1041B87A );
+    PR(G, H, A, B, C, D, E, F,   18 ,  0x0FC19DC6 );
+    PR(F, G, H, A, B, C, D, E,   19 ,  0x240CA1CC );
+    PR(E, F, G, H, A, B, C, D,   20 ,  0x2DE92C6F );
+    PR(D, E, F, G, H, A, B, C,   21 ,  0x4A7484AA );
+    PR(C, D, E, F, G, H, A, B,   22 ,  0x5CB0A9DC );
+    PR(B, C, D, E, F, G, H, A,   23 ,  0x76F988DA );
+    PR(A, B, C, D, E, F, G, H,   24 , -0x67C1AEAE );
+    PR(H, A, B, C, D, E, F, G,   25 , -0x57CE3993 );
+    PR(G, H, A, B, C, D, E, F,   26 , -0x4FFCD838 );
+    PR(F, G, H, A, B, C, D, E,   27 , -0x40A68039 );
+    PR(E, F, G, H, A, B, C, D,   28 , -0x391FF40D );
+    PR(D, E, F, G, H, A, B, C,   29 , -0x2A586EB9 );
+    PR(C, D, E, F, G, H, A, B,   30 ,  0x06CA6351 );
+    PR(B, C, D, E, F, G, H, A,   31 ,  0x14292967 );
+    PR(A, B, C, D, E, F, G, H,   32 ,  0x27B70A85 );
+    PR(H, A, B, C, D, E, F, G,   33 ,  0x2E1B2138 );
+    PR(G, H, A, B, C, D, E, F,   34 ,  0x4D2C6DFC );
+    PR(F, G, H, A, B, C, D, E,   35 ,  0x53380D13 );
+    PR(E, F, G, H, A, B, C, D,   36 ,  0x650A7354 );
+    PR(D, E, F, G, H, A, B, C,   37 ,  0x766A0ABB );
+    PR(C, D, E, F, G, H, A, B,   38 , -0x7E3D36D2 );
+    PR(B, C, D, E, F, G, H, A,   39 , -0x6D8DD37B );
+    PR(A, B, C, D, E, F, G, H,   40 , -0x5D40175F );
+    PR(H, A, B, C, D, E, F, G,   41 , -0x57E599B5 );
+    PR(G, H, A, B, C, D, E, F,   42 , -0x3DB47490 );
+    PR(F, G, H, A, B, C, D, E,   43 , -0x3893AE5D );
+    PR(E, F, G, H, A, B, C, D,   44 , -0x2E6D17E7 );
+    PR(D, E, F, G, H, A, B, C,   45 , -0x2966F9DC );
+    PR(C, D, E, F, G, H, A, B,   46 , -0x0BF1CA7B );
+    PR(B, C, D, E, F, G, H, A,   47 ,  0x106AA070 );
+    PR(A, B, C, D, E, F, G, H,   48 ,  0x19A4C116 );
+    PR(H, A, B, C, D, E, F, G,   49 ,  0x1E376C08 );
+    PR(G, H, A, B, C, D, E, F,   50 ,  0x2748774C );
+    PR(F, G, H, A, B, C, D, E,   51 ,  0x34B0BCB5 );
+    PR(E, F, G, H, A, B, C, D,   52 ,  0x391C0CB3 );
+    PR(D, E, F, G, H, A, B, C,   53 ,  0x4ED8AA4A );
+    PR(C, D, E, F, G, H, A, B,   54 ,  0x5B9CCA4F );
+    PR(B, C, D, E, F, G, H, A,   55 ,  0x682E6FF3 );
+    PR(A, B, C, D, E, F, G, H,   56 ,  0x748F82EE );
+    PR(H, A, B, C, D, E, F, G,   57 ,  0x78A5636F );
+    PR(G, H, A, B, C, D, E, F,   58 , -0x7B3787EC );
+    PR(F, G, H, A, B, C, D, E,   59 , -0x7338FDF8 );
+    PR(E, F, G, H, A, B, C, D,   60 , -0x6F410006 );
+    PR(D, E, F, G, H, A, B, C,   61 , -0x5BAF9315 );
+    PR(C, D, E, F, G, H, A, B,   62 , -0x41065C09 );
+    PR(B, C, D, E, F, G, H, A,   63 , -0x398E870E );
 
     ctx->state[0] += A;
     ctx->state[1] += B;
