@@ -1,5 +1,5 @@
 // This file has been copied 1:1 from www.spale.com/download/scrypt/scrypt1.0/sha256.c
-// Besides these 2 comment lines, nothing has been changed.
+// Besides these 2 comment lines and the correction of uint32 (see below) nothing has been changed.
 
 #ifndef _SHA256_H
 #define _SHA256_H
@@ -17,8 +17,7 @@ typedef struct
     uint32 total[2];
     uint32 state[8];
     uint8 buffer[64];
-}
-sha256_context;
+} sha256_context;
 
 void sha256_starts( sha256_context *ctx );
 void sha256_update( sha256_context *ctx, uint8 *input, uint32 length );
