@@ -9,6 +9,23 @@
 //  Module:         Thread for reading data.
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
+
 #include <errno.h>
 
 #include <QtCore>
@@ -20,7 +37,7 @@
 #include "threadwrite.h"
 
 #include "fcntl.h"
-//##define _GNU_SOURCE
+//#define _GNU_SOURCE
 
 const int THREADREAD_CHECK_CONNECTED_SLEEP = 1000;
 const int THREADREAD_SLOWDOWN_SLEEP        =  700;
@@ -29,8 +46,8 @@ const int THREADREAD_SLOWDOWN_SLEEP        =  700;
 class t_ThreadReadLocal
 {
    public:
-      t_pDevice     pDevice;
-      bool         *pSlowDownRequest;
+      t_pDevice  pDevice;
+      bool      *pSlowDownRequest;
 };
 
 t_ThreadRead::t_ThreadRead(void)
@@ -86,86 +103,67 @@ static APIRET ThreadReadCheckDeviceExists (t_pcDevice pDevice, bool &DeviceExist
    return NO_ERROR;
 }
 
-static APIRET ThreadReadDeviceDisconnected (t_pDevice pDevice)
+static APIRET ThreadReadDeviceDisconnected (t_pDevice pDevice, const char* pStrAction)
 {
-   if (pDevice->pFileSrc)
+   if (pDevice->FileDescSrc != t_Device::FileDescEmpty)
    {
-      (void) fclose (pDevice->pFileSrc);
-      pDevice->pFileSrc = NULL;
+      (void) close (pDevice->FileDescSrc);
+      pDevice->FileDescSrc = t_Device::FileDescEmpty;
    }
    if (pDevice->State == t_Device::Acquire)
         pDevice->State = t_Device::AcquirePaused;
    else pDevice->State = t_Device::VerifyPaused;
-   LOG_INFO ("Device %s disconnected, switching device state to %s", QSTR_TO_PSZ(pDevice->LinuxDevice), pDevice->StateStr())
+   LOG_INFO ("Device %s disconnected (%s, %d, %s), switching device state to %s", QSTR_TO_PSZ(pDevice->LinuxDevice), pStrAction, errno, strerror(errno), pDevice->StateStr())
 
    return ERROR_THREADREAD_DEVICE_DISCONNECTED;
 }
 
 static APIRET ThreadReadAdjustSeek (t_pDevice pDevice)        // According to the GNU Clib docu, a seek should be done after read errors
 {
-   int rc;
+   off64_t rc;
 
-   if (pDevice->pFileSrc == NULL)
+   if (pDevice->FileDescSrc == t_Device::FileDescEmpty)
       CHK (ERROR_THREADREAD_FILESRC_NOT_OPEN)
 
    if (ThreadReadEOF (pDevice))
       return NO_ERROR;
 
-   rc = fseeko64 (pDevice->pFileSrc, (off64_t)pDevice->CurrentReadPos, SEEK_SET);
-   if (rc)
-      CHK_RET (ThreadReadDeviceDisconnected (pDevice))
+   rc = lseek64 (pDevice->FileDescSrc, (off64_t)pDevice->CurrentReadPos, SEEK_SET);
+   if (rc < 0)
+      CHK_RET (ThreadReadDeviceDisconnected (pDevice, "seek"))
 
    return NO_ERROR;
 }
 
-//static APIRET ThreadReadDirectMode (FILE *pFileSrc, bool Enable)
-//{
-//   int OldFlags, NewFlags, rc;
-//   int FileDescriptor;
-//
-//   FileDescriptor = fileno (pFileSrc);
-//   if (FileDescriptor == -1)
-//      CHK (ERROR_THREADREAD_BAD_FILE_HANDLE)
-//
-//   OldFlags = fcntl (FileDescriptor, F_GETFL);
-//   if (OldFlags > 0)
-//   {
-//      NewFlags = (Enable ? (OldFlags |  O_DIRECT)
-//                         : (OldFlags & ~O_DIRECT));
-//      if (NewFlags != OldFlags)
-//      {
-//         rc = fcntl (FileDescriptor, F_SETFL, NewFlags);
-//         if (rc == -1)
-//            LOG_INFO ("Setting flags with fcntl failed. Direct mode state unknown.")
-//      }
-//      LOG_INFO ("Direct mode %s", Enable ? "enabled" : "disabled")
-//   }
-//   else
-//   {
-//      LOG_INFO ("Reading flags with fcntl failed. Direct mode remains unchanged")
-//   }
-//
-//   return NO_ERROR;
-//}
 
 static APIRET ThreadReadBlock0 (t_pDevice pDevice, unsigned char *pBuffer, unsigned int Size)
 {
-   bool DeviceExists;
-   int  Read;
+   bool    DeviceExists;
+   ssize_t Read;
+   int     OpenFlags;
 
    errno = 0;
-   if (pDevice->pFileSrc == NULL)
+   if (pDevice->FileDescSrc == t_Device::FileDescEmpty)
    {
-      pDevice->pFileSrc = fopen64 (QSTR_TO_PSZ(pDevice->LinuxDevice), "r");
-      if (!pDevice->pFileSrc)
-         CHK_RET (ThreadReadDeviceDisconnected (pDevice))
+      OpenFlags = O_RDONLY | O_NOATIME;
+      if (CONFIG (FifoMemoryManager))     // We need the buffers to be located on page size (4K) boundaries; this only
+      {                                   // is guaranteed when working with Guymager's internal memory manager (see t_FifoMemory::t_FifoMemory)
+         LOG_INFO ("Trying direct mode for %s", QSTR_TO_PSZ(pDevice->LinuxDevice))
+         OpenFlags |= O_DIRECT;
+      }
+      pDevice->FileDescSrc = open64 (QSTR_TO_PSZ(pDevice->LinuxDevice), OpenFlags);
+      if ((pDevice->FileDescSrc < 0) && (OpenFlags & O_DIRECT) && (errno == EINVAL))   // If an invalid parameter is reported, O_DIRECT might have
+      {                                                                                // caused problems. This was observed when opening files
+         OpenFlags &= ~O_DIRECT;                                                       // (added as special device) on the cases server (dd reported
+         pDevice->FileDescSrc = open64 (QSTR_TO_PSZ(pDevice->LinuxDevice), OpenFlags); // the same error with iflag=direct). Try again without it.
+      }
+      LOG_INFO ("Direct mode for %s switched %s", QSTR_TO_PSZ(pDevice->LinuxDevice), (OpenFlags & O_DIRECT) ? "on":"off")
+      if (pDevice->FileDescSrc < 0)
+         CHK_RET (ThreadReadDeviceDisconnected (pDevice, "open"))
       CHK_RET (ThreadReadAdjustSeek (pDevice))
-
-//      CHK (ThreadReadDirectMode (pDevice->pFileSrc, true))
    }
 
-   Read = (int) fread (pBuffer, Size, 1, pDevice->pFileSrc); //lint !e732 !e712 loss of sign/precision arg 2
-   // ev. zusaetzlich ferror abfragen
+   Read = read (pDevice->FileDescSrc, pBuffer, Size);
 
 // Bad sector simulation
 //   if ((pDevice->CurrentReadPos > 500000) && (pDevice->CurrentReadPos < 600000) ||
@@ -173,29 +171,23 @@ static APIRET ThreadReadBlock0 (t_pDevice pDevice, unsigned char *pBuffer, unsig
 //       (pDevice->CurrentReadPos == 0) || (pDevice->CurrentReadPos == 512) || (pDevice->CurrentReadPos == 1024))
 //      Read = 0;
 
-   switch (Read)
+   if (Read == (ssize_t)Size)
    {
-      case  1: pDevice->CurrentReadPos += Size;
-               return NO_ERROR;
-
-      case  0:
-//               LOG_INFO ("Read error at position %lld", pDevice->CurrentReadPos)
-
-               CHK (ThreadReadCheckDeviceExists (pDevice, DeviceExists))
-               if (DeviceExists)
-               {
-//                  LOG_DEBUG ("Device still exists")
-                  CHK_RET (ThreadReadAdjustSeek (pDevice))
-//                  LOG_DEBUG ("Seek adjusted")
-                  return ERROR_THREADREAD_NO_DATA;
-               }
-               else
-               {
-                  CHK_RET (ThreadReadDeviceDisconnected (pDevice))
-               }
-               break;
-
-      default: CHK_CONST (ERROR_THREADREAD_UNEXPECTED_FAILURE)
+      pDevice->CurrentReadPos += Size;
+      return NO_ERROR;
+   }
+   else
+   {
+      CHK (ThreadReadCheckDeviceExists (pDevice, DeviceExists))
+      if (DeviceExists)
+      {
+         CHK_RET (ThreadReadAdjustSeek (pDevice))
+         return ERROR_THREADREAD_NO_DATA;
+      }
+      else
+      {
+         CHK_RET (ThreadReadDeviceDisconnected (pDevice, "read"))
+      }
    }
 
    return NO_ERROR;
@@ -225,9 +217,7 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
    if (BytesToRead > INT_MAX)
       CHK (ERROR_THREADREAD_BLOCKSIZE)
 
-   if (pDevice->HasCompressionThreads())
-        CHK (t_Fifo::CreateCompressionOptimised (pFifoBlock, pDevice->FifoBlockSize))
-   else CHK (t_Fifo::Create (pFifoBlock, (unsigned int) BytesToRead))
+   CHK (t_Fifo::Create (pDevice->pFifoMemory, pFifoBlock, pDevice->FifoAllocBlockSize))
 
    pFifoBlock->DataSize = (unsigned int) BytesToRead;
    pBuffer = pFifoBlock->Buffer;
@@ -249,9 +239,8 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
             if (!pDevice->FallbackMode)
             {
                pDevice->FallbackMode = true;
-//               ReadTry = (int) std::min (pDevice->SectorSize, pDevice->SectorSizePhys);
+//               ReadTry = (int) GETMIN (pDevice->SectorSize, pDevice->SectorSizePhys);
                ReadTry = pDevice->SectorSize;
-//               CHK (ThreadReadDirectMode (pDevice->pFileSrc, true))
                LOG_INFO ("Device read error, switching %s to slow fallback mode. Reading sectors individually from now on.", QSTR_TO_PSZ(pDevice->LinuxDevice))
             }
             else
@@ -291,14 +280,14 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
    if ((pDevice->State == t_Device::AcquirePaused) ||
        (pDevice->State == t_Device::VerifyPaused ))
    {
-      CHK (t_Fifo::Destroy (pFifoBlock))
+      CHK (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlock))
       pDevice->CurrentReadPos = PrevReadPos;
 
       return ERROR_THREADREAD_DEVICE_DISCONNECTED;
    }
    else if (pDevice->AbortRequest)
    {
-      CHK (t_Fifo::Destroy (pFifoBlock))
+      CHK (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlock))
 
       return ERROR_THREADREAD_DEVICE_ABORTED;
    }
@@ -306,7 +295,6 @@ APIRET t_ThreadRead::ThreadReadBlock (t_pFifoBlock &pFifoBlock)
    {
       pDevice->FallbackMode = false;  // Switch to fast reading mode if last read was good
       LOG_INFO ("Device read ok again. Switching %s to fast block read.", QSTR_TO_PSZ(pDevice->LinuxDevice))
-//      CHK (ThreadReadDirectMode (pDevice->pFileSrc, false))
    }
 
    pFifoBlock->LastBlock = ThreadReadEOF (pDevice);
@@ -320,7 +308,8 @@ void t_ThreadRead::run (void)
    t_pFifoBlock       pFifoBlock;
    APIRET              rc;
    int                 rcf;
-   bool                CalcHashes;
+   bool                CalcHashMD5;
+   bool                CalcHashSHA256;
    quint64           *pBlocks;
    quint64             BlocksRead     = 0;
    quint64             BlocksVerified = 0;
@@ -329,22 +318,21 @@ void t_ThreadRead::run (void)
    t_HashContextMD5    HashContextMD5;
    t_HashContextSHA256 HashContextSHA256;
 
-   LOG_INFO ("Acquisition of %s: Reading thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
+   LOG_INFO ("Acquisition of %s: Read thread started", QSTR_TO_PSZ (pOwn->pDevice->LinuxDevice))
    pDevice = pOwn->pDevice;
-   CalcHashes = pDevice->Acquisition.CalcHashes && !CONFIG (UseSeparateHashThread);
+   CalcHashMD5    = pDevice->Acquisition.CalcMD5    && !CONFIG (UseSeparateHashThread);
+   CalcHashSHA256 = pDevice->Acquisition.CalcSHA256 && !CONFIG (UseSeparateHashThread);
+
    pBlocks = &BlocksRead;
    pDevice->State = t_Device::Acquire;
    for (;;)
    {
-      pDevice->pFileSrc       = NULL;
+      pDevice->FileDescSrc    = t_Device::FileDescEmpty;
       pDevice->CurrentReadPos = 0;
       pDevice->FallbackMode   = false;
       *pBlocks = 0;
-      if (CalcHashes)
-      {
-         CHK_EXIT (HashMD5Init    (&HashContextMD5   ))
-         CHK_EXIT (HashSHA256Init (&HashContextSHA256))
-      }
+      if (CalcHashMD5)    CHK_EXIT (HashMD5Init    (&HashContextMD5   ))
+      if (CalcHashSHA256) CHK_EXIT (HashSHA256Init (&HashContextSHA256))
       do
       {
          if (*(pOwn->pSlowDownRequest))
@@ -356,15 +344,13 @@ void t_ThreadRead::run (void)
             case NO_ERROR:
                pFifoBlock->Nr = (*pBlocks);
 
-               if (CalcHashes)
+               if (CalcHashMD5)    CHK_EXIT (HashMD5Append    (&HashContextMD5   , pFifoBlock->Buffer, pFifoBlock->DataSize))
+               if (CalcHashSHA256) CHK_EXIT (HashSHA256Append (&HashContextSHA256, pFifoBlock->Buffer, pFifoBlock->DataSize))
+
+               if ((pDevice->State == t_Device::Verify) && (CalcHashMD5 || CalcHashSHA256))
                {
-                  CHK_EXIT (HashMD5Append    (&HashContextMD5   , pFifoBlock->Buffer, pFifoBlock->DataSize))
-                  CHK_EXIT (HashSHA256Append (&HashContextSHA256, pFifoBlock->Buffer, pFifoBlock->DataSize))
-               }
-               if ((pDevice->State == t_Device::Verify) && CalcHashes)
-               {
-                  pDevice->IncCurrentVerifyPos(pFifoBlock->DataSize);
-                  CHK_EXIT (t_Fifo::Destroy (pFifoBlock))
+                  pDevice->IncCurrentVerifyPosSrc (pFifoBlock->DataSize);
+                  CHK_EXIT (t_Fifo::Destroy (pDevice->pFifoMemory, pFifoBlock))
                }
                else
                {
@@ -387,39 +373,38 @@ void t_ThreadRead::run (void)
          }
       } while (!ThreadReadEOF(pDevice) && !pDevice->AbortRequest);
 
+      if (pDevice->AbortRequest)
+         break;
+
       if (pDevice->State == t_Device::Acquire)
            BytesRead     = pDevice->CurrentReadPos;
       else BytesVerified = pDevice->CurrentReadPos;
 
-      if (pDevice->AbortRequest)
-         break;
-
       if (pDevice->State == t_Device::Verify)
       {
-         if (CalcHashes)
-         {
-            CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5DigestVerify   ))
-            CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256DigestVerify))
-         }
+         if (CalcHashMD5)    CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5DigestVerifySrc   ))
+         if (CalcHashSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256DigestVerifySrc))
          LOG_INFO ("Source verification completed (device %s)", QSTR_TO_PSZ (pDevice->LinuxDevice))
          break;
       }
 
-      if (CalcHashes)
-      {
-         CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5Digest   ))
-         CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256Digest))
-      }
+      if (CalcHashMD5)    CHK_EXIT (HashMD5Digest    (&HashContextMD5   , &pDevice->MD5Digest   ))
+      if (CalcHashSHA256) CHK_EXIT (HashSHA256Digest (&HashContextSHA256, &pDevice->SHA256Digest))
 
-      if (pDevice->Acquisition.VerifySource)
+      if ((pDevice->Acquisition.VerifySrc) ||
+          (pDevice->Acquisition.VerifyDst))
       {
-         (void) fclose (pDevice->pFileSrc);
-         LOG_INFO ("All data has been read (device %s), now re-reading for source verification", QSTR_TO_PSZ (pDevice->LinuxDevice))
          pDevice->State = t_Device::Verify;
          pDevice->StartTimestampVerify = QDateTime::currentDateTime();
+      }
+
+      if (pDevice->Acquisition.VerifySrc)
+      {
+         (void) close (pDevice->FileDescSrc);
+         LOG_INFO ("All data has been read (device %s), now re-reading for source verification", QSTR_TO_PSZ (pDevice->LinuxDevice))
          pBlocks = &BlocksVerified;
-         if (!CalcHashes)
-            CHK_EXIT (pDevice->pFifoRead->InsertDummy ())
+//         if (CONFIG (UseSeparateHashThread))
+         CHK_EXIT (pDevice->pFifoRead->InsertDummy ())
       }
       else
       {
@@ -428,21 +413,21 @@ void t_ThreadRead::run (void)
       }
    }
 
-   if (pDevice->pFileSrc)
+   if (pDevice->FileDescSrc != t_Device::FileDescEmpty)
    {
-      rcf = fclose (pDevice->pFileSrc);
-      pDevice->pFileSrc = NULL;
+      rcf = close (pDevice->FileDescSrc);
+      pDevice->FileDescSrc = t_Device::FileDescEmpty;
       if (rcf)
       {
-         LOG_INFO ("Unexpected error on fclose, errno = %d", errno)
+         LOG_INFO ("Unexpected error on close, errno = %d", errno)
          pDevice->AbortReason  = t_Device::ThreadReadFileError;
          pDevice->AbortRequest = true;
       }
    }
 
-   LOG_INFO ("Reading thread exits now (device %s, %Ld blocks read, %llu bytes)", QSTR_TO_PSZ (pDevice->LinuxDevice), BlocksRead, BytesRead)
-   if (pDevice->Acquisition.VerifySource)
-   LOG_INFO ("                    (for the verification: %Ld blocks read, %llu bytes)", BlocksVerified, BytesVerified)
+   LOG_INFO ("Read thread exits now (device %s, %Ld blocks read, %llu bytes)", QSTR_TO_PSZ (pDevice->LinuxDevice), BlocksRead, BytesRead)
+   if (pDevice->Acquisition.VerifySrc)
+   LOG_INFO ("             (for the source verification: %Ld blocks read, %llu bytes)", BlocksVerified, BytesVerified)
 }
 
 void t_ThreadRead::SlotFinished (void)
