@@ -9,6 +9,22 @@
 //  Module:         Thread for writing data.
 // ****************************************************************************
 
+// Copyright 2008, 2009, 2010 Guy Voncken
+//
+// This file is part of guymager.
+//
+// guymager is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// guymager is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with guymager. If not, see <http://www.gnu.org/licenses/>.
 
 #ifndef __THREADWRITE_H__
 #define __THREADWRITE_H__
@@ -31,8 +47,10 @@ class t_ThreadWrite: public QThread
       t_ThreadWrite (t_pDevice pDevice, bool *pSlowDownRequest);
      ~t_ThreadWrite ();
 
-      APIRET GetpFileHandle0 (void **ppHandle);
-      APIRET GetpFileHandle  (void **ppHandle);
+      APIRET GetpFileHandle    (void **ppHandle);
+      APIRET ReleaseFileHandle (void);
+   private:
+      APIRET GetpFileHandle0   (void **ppHandle);
 
    protected:
       void run (void);
@@ -41,7 +59,8 @@ class t_ThreadWrite: public QThread
       APIRET DeleteImageFiles (bool AlsoDeleteInfoFile);
 
    signals:
-      void SignalEnded (t_pDevice pDevice);
+      void SignalEnded        (t_pDevice pDevice);
+      void SignalFreeMyHandle (t_pDevice pDevice);
 
    private slots:
       void SlotFinished (void);
@@ -58,6 +77,7 @@ enum
 {
    ERROR_THREADWRITE_OPEN_FAILED = ERROR_BASE_THREADWRITE + 1,
    ERROR_THREADWRITE_WRITE_FAILED,
+   ERROR_THREADWRITE_VERIFY_FAILED,
    ERROR_THREADWRITE_CLOSE_FAILED,
    ERROR_THREADWRITE_NOT_OPENED,
    ERROR_THREADWRITE_INVALID_FORMAT,
@@ -66,7 +86,8 @@ enum
    ERROR_THREADWRITE_OUT_OF_SEQUENCE_BLOCK,
    ERROR_THREADWRITE_HANDLE_NOT_YET_AVAILABLE,
    ERROR_THREADWRITE_HANDLE_TIMEOUT,
-   ERROR_THREADWRITE_LIBEWF_FAILED
+   ERROR_THREADWRITE_LIBEWF_FAILED,
+   ERROR_THREADWRITE_MALLOC_FAILED,
 };
 
 #endif
